from insights.core.exceptions import SkipComponent
from insights.parsers import nmap
from insights.parsers.nmap import NmapSsh
from insights.tests import context_wrap
import doctest
import pytest

NMAP_SSH_STYLE1 = """
Starting Nmap 7.92 ( https://nmap.org ) at 2024-05-03 01:52 UTC
Nmap scan report for localhost (127.0.0.1)
Host is up (0.00024s latency).

PORT   STATE SERVICE VERSION
22/tcp open  ssh     OpenSSH 8.0 (protocol 2.0)
| ssh2-enum-algos:
|   kex_algorithms: (11)
|       curve25519-sha256
|       curve25519-sha256@libssh.org
|       ecdh-sha2-nistp256
|       ecdh-sha2-nistp384
|       ecdh-sha2-nistp521
|       diffie-hellman-group-exchange-sha256
|       diffie-hellman-group14-sha256
|       diffie-hellman-group16-sha512
|       diffie-hellman-group18-sha512
|       diffie-hellman-group-exchange-sha1
|       diffie-hellman-group14-sha1
|   server_host_key_algorithms: (1)
|       rsa-sha2-512
|   encryption_algorithms: (5)
|       aes256-gcm@openssh.com
|       chacha20-poly1305@openssh.com
|       aes256-ctr
|       aes128-gcm@openssh.com
|       aes128-ctr
|   mac_algorithms: (8)
|       hmac-sha2-256-etm@openssh.com
|       hmac-sha1-etm@openssh.com
|       umac-128-etm@openssh.com
|       hmac-sha2-512-etm@openssh.com
|       hmac-sha2-256
|       hmac-sha1
|       umac-128@openssh.com
|       hmac-sha2-512
|   compression_algorithms: (2)
|       none
|_      zlib@openssh.com

Service detection performed. Please report any incorrect results at https://nmap.org/sub|       curve25519-sha256@libssh.org
mit/ .
Nmap done: 1 IP address (1 host up) scanned in 1.03 seconds
""".strip()

NMAP_SSH_STYLE2 = """
Starting Nmap 6.40 ( http://nmap.org ) at 2024-07-07 01:33 EDT
Nmap scan report for localhost.localdomain (127.0.0.1)
Host is up (0.000048s latency).
PORT   STATE SERVICE VERSION
22/tcp open  ssh     OpenSSH 7.4 (protocol 2.0)
| ssh2-enum-algos:
|   kex_algorithms (12)
|       curve25519-sha256
|       curve25519-sha256@libssh.org
|       ecdh-sha2-nistp256
|       ecdh-sha2-nistp384
|       ecdh-sha2-nistp521
|       diffie-hellman-group-exchange-sha256
|       diffie-hellman-group16-sha512
|       diffie-hellman-group18-sha512
|       diffie-hellman-group-exchange-sha1
|       diffie-hellman-group14-sha256
|       diffie-hellman-group14-sha1
|       diffie-hellman-group1-sha1
|   server_host_key_algorithms (5)
|       ssh-rsa
|       rsa-sha2-512
|       rsa-sha2-256
|       ecdsa-sha2-nistp256
|       ssh-ed25519
|   encryption_algorithms (5)
|       aes128-ctr
|       aes192-ctr
|       aes256-ctr
|       aes128-gcm@openssh.com
|       aes256-gcm@openssh.com
|   mac_algorithms (4)
|       umac-64@openssh.com
|       umac-128@openssh.com
|       hmac-sha2-256
|       hmac-sha2-512
|   compression_algorithms (2)
|       none
|_      zlib@openssh.com

Service detection performed. Please report any incorrect results at http://nmap.org/submit/ .
Nmap done: 1 IP address (1 host up) scanned in 0.55 seconds%
""".strip()

NMAP_SSH_ERROR1 = """
Starting Nmap 7.92 ( https://nmap.org ) at 2024-05-03 02:30 UTC
Nmap scan report for localhost (127.0.0.1)
Host is up (0.00015s latency).

PORT   STATE  SERVICE   VERSION
24/tcp closed priv-mail

Service detection performed. Please report any incorrect results at https://nmap.org/submit/ .
Nmap done: 1 IP address (1 host up) scanned in 0.96 seconds
""".strip()


def test_nmap_ssh():
    results = NmapSsh(context_wrap(NMAP_SSH_STYLE1))
    assert len(results) == 5
    assert len(results["mac_algorithms"]) == 8
    assert results["server_host_key_algorithms"] == ["rsa-sha2-512"]

    results = NmapSsh(context_wrap(NMAP_SSH_STYLE2))
    assert len(results) == 5
    assert len(results["mac_algorithms"]) == 4
    assert results["server_host_key_algorithms"][0] == "ssh-rsa"

    with pytest.raises(SkipComponent) as e:
        NmapSsh(context_wrap(NMAP_SSH_ERROR1))
    assert "The output format is not expected" in str(e)


def test_repquota_doc_examples():
    env = {
        'nmap_ssh': NmapSsh(context_wrap(NMAP_SSH_STYLE1))
    }
    failed, total = doctest.testmod(nmap, globs=env)
    assert failed == 0



import logging
import os
import shlex
import signal
import six
import sys

from subprocess import Popen, PIPE, STDOUT

from insights.core.exceptions import CalledProcessError
from insights.util import which

try:
    from subprocess import DEVNULL
except:
    DEVNULL = open(os.devnull, 'w')

log = logging.getLogger(__name__)


class Pipeline(object):
    """
    Connect a list of lists of commands together with the stdout of one as the
    stdin of the next. The output of the last command is written to out_stream.

    >>> p = Pipeline("ls -lrt", "grep .py")
    >>> output = p()
    >>> p.write("pythons.txt")
    """

    def __init__(self, *cmds, **kwargs):
        """
        cmds (list): one or more commands. Each command will be shlex.split if
            it isn't already split.
        bufsize (int): size of the I/O buffers to use between processes running
            the commands. -1 means to use system defaults. 0 means no buffering.
            Defaults to -1.
        env (dict): environment in which to execute commands. Defaults to
            os.environ.
        timeout (int): number of seconds to wait before killing the command.
            Defaults to None, which waits forever.
        signum (int): signal to send the command on timeout. Defaults to
            signal.SIGKILL
        """

        self.bufsize = kwargs.get("bufsize", -1)
        self.env = kwargs.get("env", os.environ)
        max_failure_output_env = self.env.get("MAX_FAILURE_OUTPUT", "1024")
        try:
            max_failure_output = int(max_failure_output_env)
            if max_failure_output <= 0:
                raise ValueError
        except (ValueError, TypeError):
            max_failure_output = 1024
        self.max_failure_output = max_failure_output
        timeout = kwargs.get("timeout")
        signum = kwargs.get("signum", signal.SIGKILL)

        cmds = [shlex.split(c) if not isinstance(c, list) else c for c in cmds]
        timeout_command = which("timeout", env=self.env)
        if timeout:
            if timeout_command:
                to = shlex.split("timeout -s {0} {1}".format(signum, timeout))
                to.extend(cmds[0])
                cmds[0] = to
            else:
                # TODO: Should this raise an exception instead?
                log.warn("Timeout specified but timeout command unavailable.")
        self.cmds = cmds

    def _build_pipes(self, out_stream=PIPE):
        log.debug("Executing: %s" % str(self.cmds))
        if len(self.cmds) == 1:
            return Popen(
                self.cmds[0],
                bufsize=self.bufsize,
                stdin=DEVNULL,
                stderr=STDOUT,
                stdout=out_stream,
                env=self.env,
            )

        stdout = Popen(
            self.cmds[0],
            bufsize=self.bufsize,
            stdin=DEVNULL,
            stderr=STDOUT,
            stdout=PIPE,
            env=self.env,
        ).stdout
        last = len(self.cmds) - 2
        for i, arg in enumerate(self.cmds[1:]):
            if i < last:
                stdout = Popen(
                    arg,
                    bufsize=self.bufsize,
                    stdin=stdout,
                    stderr=STDOUT,
                    stdout=PIPE,
                    env=self.env,
                ).stdout
            else:
                return Popen(
                    arg,
                    bufsize=self.bufsize,
                    stdin=stdout,
                    stderr=STDOUT,
                    stdout=out_stream,
                    env=self.env,
                )

    def __call__(self, keep_rc=False):
        """
        Executes the pipeline.

        Returns:
            The final output of the pipeline if keep_rc is False;
            an (exit code, output) tuple if keep_rc is True.
        Raises:
            CalledProcessError if any return code in the pipeline is nonzero
            and keep_rc is False.
        """
        p = self._build_pipes()
        output = p.communicate()[0]
        rc = p.poll()
        if keep_rc:
            return (rc, output)
        if rc:
            # it's enough for trobuleshooting to keep the first 1024 charactersof the failure output
            raise CalledProcessError(rc, self.cmds[0], output[: self.max_failure_output])
        return output

    def write(self, output, mode="w", keep_rc=False):
        """
        Executes the pipeline and writes the results to the supplied output.
        If output is a filename and the file didn't already exist before trying
        to write, the file will be removed if an exception is raised.

        Args:
            output (str or file like object): will create a new file of this
                name or overwrite an existing file. If output is already a file
                like object, it is used.
            mode (str): mode to use when creating or opening the provided file
                name if it is a string. Ignored if output is a file like object.

        Returns:
            The final output of the pipeline.
        Raises:
            CalledProcessError if any return code in the pipeline is nonzero.
        """
        if isinstance(output, six.string_types):
            already_exists = os.path.exists(output)
            try:
                with open(output, mode) as f:
                    p = self._build_pipes(f)
                    rc = p.wait()
                    if keep_rc:
                        return rc
                    if rc:
                        raise CalledProcessError(rc, self.cmds[0], "")
            except BaseException as be:
                if not already_exists and os.path.exists(output):
                    os.remove(output)
                six.reraise(be.__class__, be, sys.exc_info()[2])
        else:
            p = self._build_pipes(output)
            rc = p.wait()
            if keep_rc:
                return rc
            if rc:
                raise CalledProcessError(rc, self.cmds[0], "")


def call(cmd, timeout=None, signum=signal.SIGKILL, keep_rc=False, encoding="utf-8", env=os.environ):
    """
    Execute a cmd or list of commands with an optional timeout in seconds.

    If `timeout` is supplied and expires, the process is killed with
    SIGKILL (kill -9) and an exception is raised. Otherwise, the command
    output is returned.

    Parameters
    ----------
    cmd: str or [[str]]
        The command(s) to execute
    timeout: int
        Seconds before kill is issued to the process
    signum: int
        The signal number to issue to the process on timeout
    keep_rc: bool
        Whether to return the exit code along with the output
    encoding: str
        unicode decoding scheme to use. Default is "utf-8"
    env: dict
        The environment in which to execute commands. Default is os.environ

    Returns
    -------
    str
        Content of stdout of cmd on success.

    Raises
    ------
        CalledProcessError
            Raised when cmd fails
    """

    if not isinstance(cmd, list):
        cmd = [cmd]

    signum = signum or signal.SIGKILL

    p = Pipeline(*cmd, timeout=timeout, signum=signum, env=env)
    res = p(keep_rc=keep_rc)

    if keep_rc:
        rc, output = res
        output = output.decode(encoding, 'ignore')
        return rc, output
    return res.decode(encoding, "ignore")



"""
Memory controller information of cgroup - ``/sys/fs/cgroup/memeory``
====================================================================

Parser included in this module is:

SysFsCgroupUniqMemorySwappiness - Datasource ``sys_fs_cgroup_uniq_memory_swappiness``
-------------------------------------------------------------------------------------
"""
from collections import namedtuple

from insights.core import Parser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.sys_fs_cgroup_uniq_memory_swappiness)
class SysFsCgroupUniqMemorySwappiness(Parser):
    """
    Class to parse the datasource ``sys_fs_cgroup_uniq_memory_swappiness`` output.

    Sample output of datasource looks like::
        10   1
        60   66

    The two columns that are output are:
    1. value - the value of memory.swappiness
    2. count - how many memory.swappiness files have the same setting

    Examples:
        >>> type(sys_fs_cgroup_uniq_memory_swappiness)
        <class 'insights.parsers.sys_fs_cgroup_memory_swappiness.SysFsCgroupUniqMemorySwappiness'>
        >>> sys_fs_cgroup_uniq_memory_swappiness.stats[0]
        MemorySwappiness(count=1, value=10)
        >>> sys_fs_cgroup_uniq_memory_swappiness.stats[0].count
        1
        >>> sys_fs_cgroup_uniq_memory_swappiness.stats[1].value
        60

    Attributes:
        stats (list): a list of ``MemorySwappiness`` objects
    """

    MemorySwappiness = namedtuple('MemorySwappiness', ['value', 'count'])
    """namedtuple: Structure to hold a line of ``sys_fs_cgroup_uniq_memory_swappiness`` datasource."""

    def parse_content(self, content):
        self.stats = []
        for line in content:
            parts = line.split()
            self.stats.append(self.MemorySwappiness(int(parts[0]), int(parts[1])))



import doctest
import pytest
from insights import SkipComponent
from insights.parsers import client_metadata
from insights.parsers.client_metadata import (AnsibleHost, BlacklistedSpecs,
                                              BranchInfo, DisplayName,
                                              MachineID, Tags, VersionInfo)
from insights.tests import context_wrap
from insights.tests.parsers import skip_component_check

ANSIBLE_HOST = "host1"

TAGS_JSON_CONTENT = """
[{"key": "group", "value": "_group-name-value_", "namespace": "insights-client"}, {"key": "location", "value": "_location-name-value_", "namespace": "insights-client"}]
""".strip()

BLACKLISTED_SPECS = '{"specs": ["dmesg", "fstab"]}'

BRANCH_INFO = """{"remote_branch": -1, "remote_leaf": -1}"""

MACHINE_ID = """
176843d1-90fa-499b-9f94-111111111111
""".strip()

VER_INFO_1 = """
{"core_version": "3.0.8-dev", "client_version": "3.1.1"}
""".strip()

VER_INFO_2 = """
{"core_version": "3.0.203-1", "client_version": "3.1.1"}
""".strip()


def test_ansible_host():
    ret = AnsibleHost(context_wrap(ANSIBLE_HOST))
    assert ret.hostname == 'host1'


def test_blacklisted_specs():
    bs = BlacklistedSpecs(context_wrap(BLACKLISTED_SPECS))
    assert bs.specs[0] == "dmesg"
    assert bs.specs[1] == "fstab"


def test_blacklisted_spec_skip():
    with pytest.raises(SkipComponent) as ex:
        BlacklistedSpecs(context_wrap(""))
    assert "Empty output." in str(ex)


def test_branch_info():
    ret = BranchInfo(context_wrap(BRANCH_INFO))
    assert ret.data['remote_branch'] == -1
    assert ret.data['remote_leaf'] == -1


def test_display_name():
    ret = DisplayName(context_wrap(ANSIBLE_HOST))  # test with ANSIBLE_HOST
    assert ret.hostname == 'host1'
    assert ret.raw == 'host1'


def test_machine_id():
    ret = MachineID(context_wrap(MACHINE_ID))
    assert ret.id == '176843d1-90fa-499b-9f94-111111111111'

    with pytest.raises(SkipComponent):
        MachineID(context_wrap(""))


def test_tags_json():
    ret = Tags(context_wrap(TAGS_JSON_CONTENT))
    assert ret.data[0]['key'] == "group"
    assert ret.data[0]['value'] == "_group-name-value_"
    assert ret.data[0]['namespace'] == "insights-client"
    assert ret.data[1]['key'] == "location"
    assert ret.data[1]['value'] == "_location-name-value_"
    assert ret.data[1]['namespace'] == "insights-client"


def test_tags_json_bytes():
    ret = Tags(context_wrap(bytes(str(TAGS_JSON_CONTENT).encode("utf-8"))))
    assert ret.data[0]['key'] == "group"
    assert ret.data[0]['value'] == "_group-name-value_"
    assert ret.data[0]['namespace'] == "insights-client"
    assert ret.data[1]['key'] == "location"
    assert ret.data[1]['value'] == "_location-name-value_"
    assert ret.data[1]['namespace'] == "insights-client"


def test_tags_empty():
    assert 'Empty output.' in skip_component_check(Tags)


def test_version_info():
    ret = VersionInfo(context_wrap(VER_INFO_1))
    assert ret.core_version == '3.0.8-dev'
    assert ret.client_version == '3.1.1'

    ret = VersionInfo(context_wrap(VER_INFO_2))
    assert ret.core_version == '3.0.203-1'
    assert ret.client_version == '3.1.1'


def test_version_info_empty():
    assert 'Empty output.' in skip_component_check(VersionInfo)


def test_doc_examples():
    env = {
        "ansible_host": AnsibleHost(context_wrap(ANSIBLE_HOST)),
        "specs": BlacklistedSpecs(context_wrap(BLACKLISTED_SPECS)),
        "branch_info": BranchInfo(context_wrap(BRANCH_INFO)),
        "display_name": DisplayName(context_wrap(ANSIBLE_HOST)),  # test with ANSIBLE_HOST
        "machine_id": MachineID(context_wrap(MACHINE_ID)),
        "tags": Tags(context_wrap(TAGS_JSON_CONTENT)),
        'ver': VersionInfo(context_wrap(VER_INFO_2)),
    }
    failed, total = doctest.testmod(client_metadata, globs=env)
    assert failed == 0



"""
Custom datasources for mdadm information
"""
import glob
import os
import stat

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource


@datasource(HostContext)
def raid_devices(broker):
    """ Glob extent "/dev/md*" to pass to ``mdadm_D`` Spec. """
    try:
        extended = glob.glob("/dev/md*")
        devices = [x for x in extended if stat.S_ISBLK(os.stat(x).st_mode)]
        if devices:
            return ' '.join(devices)
        else:
            raise SkipComponent("No /dev/md* raid devices found")
    except Exception as e:
        raise SkipComponent("Skipping raid_devices: {0}".format(str(e)))



# -*- coding: utf-8 -*-
"""
Utility functions of hostname
=============================
"""
import socket


def determine_hostname(display_name=None):
    """
    Find fqdn if we can
    """
    if display_name:
        # if display_name is provided, just return the given name
        return display_name
    else:
        socket_gethostname = socket.gethostname()
        socket_fqdn = socket.getfqdn()

        try:
            socket_ex = socket.gethostbyname_ex(socket_gethostname)[0]
        except (LookupError, socket.gaierror):
            socket_ex = ''

        gethostname_len = len(socket_gethostname)
        fqdn_len = len(socket_fqdn)
        ex_len = len(socket_ex)

        if fqdn_len > gethostname_len or ex_len > gethostname_len:
            if "localhost" not in socket_ex and len(socket_ex):
                return socket_ex
            if "localhost" not in socket_fqdn:
                return socket_fqdn

        return socket_gethostname



"""
OpenVmTools - Commands ``open-vm-tools``
========================================

Parsers that parse the output of command ``open-mv-tools`` are included in this
module:

OpenVmToolsStatRawTextSession - Command ``vmware-toolbox-cmd stat raw text session``
------------------------------------------------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.open_vm_tools_stat_raw_text_session)
class OpenVmToolsStatRawTextSession(CommandParser, dict):
    """
    Class to parse the output of command ``vmware-toolbox-cmd stat raw text session``

    Sample input::

        session = 4004861987670969122
        uptime = 1036293956
        version = VMware ESXi 6.0.0 build-12345
        provider =
        uuid.bios = 00 00 00 00 00 00 66 8e-00 00 00 00 51 1e 23 f3

    Examples:
        >>> type(ovmt)
        <class 'insights.parsers.open_vm_tools.OpenVmToolsStatRawTextSession'>
        >>> ovmt['version'] == 'VMware ESXi 6.0.0 build-12345'
        True
    """

    def parse_content(self, content):
        if not content or 'must be run inside a virtual machine' in content[0]:
            raise SkipComponent

        data = dict()
        for line in content:
            if '=' in line:
                key, value = [i.strip() for i in line.split('=', 1)]
                data[key] = value

        if not data:
            raise SkipComponent

        self.update(data)



from mock.mock import patch
from pytest import mark

from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner


@mark.parametrize(
    ("line", "expected"),
    [
        ("test_no_ip", "test_no_ip"),
        ("test 127.0.0.1", "test 127.0.0.1"),
        ("radius_ip_1=10.0.0.1", "radius_ip_1=10.230.230.1"),
        (
            (
                "        inet 10.0.2.15"
                "  netmask 255.255.255.0"
                "  broadcast 10.0.2.255"
                " dup 10.0.2.15"
            ),
            (
                "        inet 10.230.230.3"
                "  netmask 10.230.230.1"
                "  broadcast 10.230.230.2"
                " dup 10.230.230.3"
            ),
        ),
        (
            ["inet 10.0.2.15", "  netmask 255.255.255.0", " broadcast 10.0.2.255", "dup 10.0.2.15"],
            [
                "inet 10.230.230.1",
                "  netmask 10.230.230.3",
                " broadcast 10.230.230.2",
                "dup 10.230.230.1",
            ],
        ),
        (
            "radius_ip_1=10.0.0.100-10.0.0.200",
            "radius_ip_1=10.230.230.1-10.230.230.2",
        ),
    ],
)
@mark.parametrize(
    ("obfuscate", "obfuscation_list"),
    [
        (True, None),
        (None, ['ipv4']),
    ],
)
def test_obfuscate_ipv4_match(obfuscate, obfuscation_list, line, expected):
    c = InsightsConfig(obfuscate=obfuscate, obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    actual = pp.clean_content(line)
    assert actual == expected


@mark.parametrize(
    ("line", "expected"),
    [
        (
            ("        inet 10.0.2.155" "  netmask 10.0.2.1" "  broadcast 10.0.2.15"),
            ("        inet 10.230.230.1" "  netmask 10.230.230.3" "  broadcast 10.230.230.2"),
        ),
    ],
)
@mark.parametrize(
    ("obfuscate", "obfuscation_list"),
    [
        (True, None),
        (None, ['ipv4']),
    ],
)
def test_obfuscate_ipv4_match_IP_overlap(obfuscate, obfuscation_list, line, expected):
    c = InsightsConfig(obfuscate=obfuscate, obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    actual = pp.clean_content(line)
    assert actual == expected


@mark.parametrize(
    ("line", "expected"),
    [
        ("test_no_ip", "test_no_ip"),
        ("test 127.0.0.1", "test 127.0.0.1"),
        (
            "tcp6       0      0 100.100.100.101:23    10.231.200.1:63564 ESTABLISHED 0",
            "tcp6       0      0 10.230.230.1:23       10.230.230.2:63564 ESTABLISHED 0",
        ),
        (
            "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0",
            "tcp6       0      0 10.230.230.2:23       10.230.230.1:63564 ESTABLISHED 0",
        ),
        (
            "tcp6  10.0.0.11    0 10.0.0.1:23       10.0.0.111:63564    ESTABLISHED 0",
            "tcp6  10.230.230.2 0 10.230.230.3:23   10.230.230.1:63564  ESTABLISHED 0",
        ),
        (
            "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.0.1\n",
            "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1\n",
        ),
        (
            "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.111.11\n",
            "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1 \n",
        ),
    ],
)
@mark.parametrize(
    ("obfuscate", "obfuscation_list"),
    [
        (True, None),
        (None, ['ipv4']),
    ],
)
def test_obfuscate_ipv4_match_IP_overlap_netstat(obfuscate, obfuscation_list, line, expected):
    c = InsightsConfig(obfuscate=obfuscate, obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    actual1 = pp.clean_content(line, width=True)
    assert actual1 == expected


@mark.parametrize(
    ("original", "expected"),
    [
        (
            "{\"name\":\"shadow-utils\","
            "\"epoch\":\"2\","
            "\"version\":\"4.1.5.1\","
            "\"release\":\"5.el6\","
            "\"arch\":\"x86_64\","
            "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
            "\"buildtime\":\"1455012203\","
            "\"vendor\":\"Red Hat, Inc.\","
            "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
            "\"sigpgp\":"
            "\"RSA/8, "
            "Tue 08 Mar 2016 11:15:08 AM CET, "
            "Key ID 199e2f91fd431d51\"}",
            "{\"name\":\"shadow-utils\","
            "\"epoch\":\"2\","
            "\"version\":\"10.230.230.1\","
            "\"release\":\"5.el6\","
            "\"arch\":\"x86_64\","
            "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
            "\"buildtime\":\"1455012203\","
            "\"vendor\":\"Red Hat, Inc.\","
            "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
            "\"sigpgp\":"
            "\"RSA/8, "
            "Tue 08 Mar 2016 11:15:08 AM CET, "
            "Key ID 199e2f91fd431d51\"}",
        )
    ],
)
@mark.parametrize(
    ("obfuscate", "obfuscation_list"),
    [
        (True, None),
        (None, ['ipv4']),
    ],
)
@patch("insights.cleaner.ip.IPv4._ip2db", return_value="10.230.230.1")
def test_obfuscate_ipv4_false_positive(_ip2db, obfuscate, obfuscation_list, original, expected):
    c = InsightsConfig(obfuscate=obfuscate, obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    actual = pp.clean_content(original)
    assert actual == expected
    # "no_obfuscate=['ipv4']
    actual = pp.clean_content(original, no_obfuscate=['ipv4'])
    assert actual == original



import os

from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner

test_file_data = 'test\nabcd\n1234\npwd: p4ssw0rd\n'


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_keyword_empty_not_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {})  # empty keywords
    pp.clean_file(test_file, [])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_keyword_changed_keyword(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': ['test']})
    pp.clean_file(test_file, [])
    # file is changed
    with open(test_file, 'r') as t:
        data = t.readlines()
        assert 'test' not in data[0]
        assert 'keyword0' in data[0]
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_keyword_no_such_keyword_to_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': ['t_e_s_t']})  # no such keyword
    pp.clean_file(test_file, [])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_keyword_disabled_by_no_obfuacate(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': 'test'})
    pp.clean_file(test_file, ['keyword'])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()



import re
import string

from .. import Parser, parser, LegacyItemAccess
from insights.specs import Specs
from insights.parsr import (Char, EOF, HangingString, InSet, Many, OneLineComment, Opt,
                            skip_none, String, WithIndent, WS)


header_chars = (set(string.printable) - set(string.whitespace) - set("[]")) | set(" ")
sep_chars = set(":=")
key_chars = header_chars - sep_chars - set(" ")
value_chars = set(string.printable) - set("\n\r")

LeftEnd = WS >> Char("[") << WS
RightEnd = WS >> Char("]") << WS
Header = LeftEnd >> String(header_chars) << RightEnd
Key = WS >> String(key_chars) << WS
Sep = InSet(sep_chars)
Value = WS >> HangingString(value_chars)
KVPair = WithIndent(Key + Opt(Sep >> Value))
Comment = WS >> (OneLineComment("#") | OneLineComment(";")).map(lambda x: None)

Line = Comment | KVPair.map(tuple)
Sect = (Header + Many(Line).map(skip_none).map(dict)).map(tuple)
Doc = Many(Comment | Sect).map(skip_none).map(dict)
Top = Doc << WS << EOF


def parse_yum_repos(content):
    doc = Top(content)
    for k, v in doc.items():
        for special in ("baseurl", "gpgkey"):
            if special in v:
                v[special] = [i.strip() for i in re.split(",| ", v[special])]
    return doc


@parser(Specs.yum_repos_d)
class YumReposD(LegacyItemAccess, Parser):
    """Class to parse the files under ``yum.repos.d`` """

    def get(self, key):
        return self.data.get(key)

    def parse_content(self, content):
        """
        Return an object contains a dict.
        {
            "rhel-source": {
                "gpgcheck": "1",
                "gpgkey": ["file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release",
                           "file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release_bak"]
                "enabled": "0",
                "name": "Red Hat Enterprise Linux $releasever - $basearch - Source",
                "baseurl": "ftp://ftp.redhat.com/pub/redhat/linux/enterprise/$releasever/en/os/SRPMS/"
            }
        }
        ----------------------------------------------------
        There are several files in 'yum.repos.d' directory, which have the same
        format.  For example:
        --------one of the files : rhel-source.repo---------
        [rhel-source]
        name=Red Hat Enterprise Linux $releasever - $basearch - Source
        baseurl=ftp://ftp.redhat.com/pub/redhat/linux/enterprise/$releasever/en/os/SRPMS/
        enabled=0
        gpgcheck=1
        gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
               file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release_bak
        """
        self.data = parse_yum_repos("\n".join(content))

    def __iter__(self):
        for repo in self.data:
            yield repo



"""
FirewallDConf - file ``/etc/firewalld/firewalld.conf``
======================================================

The FirewallDConf class parses the file ``/etc/firewalld/firewalld.conf``.
And returns a dict contains the firewall configurations.

Examples:

    >>> type(firewalld)
    <class 'insights.parsers.firewall_config.FirewallDConf'>
    >>> 'DefaultZone' in firewalld
    True
    >>> firewalld['DefaultZone']
    'public'
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.specs import Specs


@parser(Specs.firewalld_conf)
class FirewallDConf(Parser, dict):
    """
    Class for parsing ``/etc/firewalld/firewalld.conf`` file.
    """

    def parse_content(self, content):
        self.update(split_kv_pairs(content, use_partition=False))
        if not self:
            raise SkipComponent("empty content")



"""
Custom datasources provide functionality beyond ``simple_file`` and ``simple_command``
type datasources.  If the custom datasource is short and concise it may be added to
this module.  Other datasources should be added as a separate module.
Normally custom datasources are necessary for core-collection.  In order for a custom
datasource to execute, all of its dependencies must be explicitly loaded by the client.
The client uses the YAML template :py:data:`insights.collect.default_manifest` and each
parser/combiner/component required by a custom datasource must be included in the YAML
template to ensure it is loaded.
"""

import os
import time

DEFAULT_SHELL_TIMEOUT = 10
""" int: Default timeout in seconds for ctx.shell_out() commands, must be provided as an arg """


def get_running_commands(ps, ctx, commands):
    """
    Search for a list of commands in Ps combiner output and returns the full path
    to the command

    Arguments:
        ps: Ps combiner object
        ctx: Context of the current collection
        commands(list): List of commands to search for in ps output

    Returns:
        list: List of the full command paths of the all ``command``.

    Raises:
        TypeError: Raised when ``commands`` args is not a list
    """
    if not commands or not isinstance(commands, list):
        raise TypeError('Commands argument must be a list object and contain at least one item')

    ps_list = [ps.search(COMMAND_NAME__contains=c) for c in commands]
    if ps_list and isinstance(ps_list[0], list):
        ps_cmds = [i for sub_l in ps_list for i in sub_l]
    else:
        ps_cmds = ps_list

    ret = set()
    for cmd in set(p['COMMAND'] for p in ps_cmds):
        try:
            cmd_prefix = cmd.split(None, 1)[0]
            which = ctx.shell_out(
                "/usr/bin/which {0}".format(cmd_prefix), timeout=DEFAULT_SHELL_TIMEOUT
            )
        except Exception:
            continue
        ret.add(which[0]) if which else None
    return sorted(ret)


def get_recent_files(target_path, last_modify_hours=24, latest_count=0):
    """
    Get the recent updated or created files, limited to lastest_count files

    Arguments:
        target_path (string): target path to search
        last_modify_hours (int): specify the recent hours
        latest_count (int): specify the lastest files count with a positive integer

    Returns:
        list: files that updated or creates just in last_modify_hours hours
    """
    latest_files = {}
    if os.path.exists(target_path):
        current_time = time.time()
        for parent_path, _, report_files in os.walk(target_path):
            for one_file in report_files:
                t_full_file = os.path.join(parent_path, one_file)
                file_time = os.path.getmtime(t_full_file)
                if (current_time - file_time) // 3600 < last_modify_hours:
                    latest_files[t_full_file] = file_time
    result_files = sorted(latest_files, key=lambda x: latest_files[x], reverse=True)
    return result_files[:latest_count] if latest_count > 0 else result_files



"""
Link Layer stats
================

Combiner for link layer stats. It uses the results of
the ``netstat -i`` parser and the ``ip -s link`` parser to determine the
network stats of link layer. ``ip -s link`` is the preferred source
of data and return object which can be used to access ``group_by_iface``.

Examples:
    >>> type(nstat)
    <class 'insights.combiners.netstat.NetworkStats'>
    >>> stats = nstat.group_by_iface
    >>> stats["lo"]
    {'RX-OK': 98, 'TX-OK': 100, 'MTU': 65536, 'RX-ERR': 0, 'TX-DRP': 0, 'TX-ERR': 0, 'RX-DRP': 0, 'RX-OVR': 0, 'Flg': 'LRU'}
    >>> stats["enp0s8"]
    {'RX-OK': 6, 'TX-DRP': 0, 'TX-OK': 4, 'MTU': 1500, 'RX-ERR': 0, 'TX-ERR': 0, 'RX-DRP': 0, 'RX-OVR': 0, 'Flg': 'BMRU'}
    >>> print nstat.data[0]['Iface']
    enp0s8
    >>> for dev_item in nstat.data:
    >>>     print dev_item
    {'RX-OK': '842447', 'TX-OVR': '0', 'Iface': 'bond1', 'TX-OK': '4233', 'MTU': '1500', 'Met': '0', 'RX-ERR': '0', 'TX-DRP': '0', 'TX-ERR': '0', 'RX-DRP': '0', 'RX-OVR': '0', 'Flg': 'BMmRU'}
    {'RX-OK': '422518', 'TX-OVR': '0', 'Iface': 'eth0', 'TX-OK': '1703', 'MTU': '1500', 'Met': '0', 'RX-ERR': '0', 'TX-DRP': '0', 'TX-ERR': '0', 'RX-DRP': '0', 'RX-OVR': '0', 'Flg': 'BMsRU'}
    ...
    ...
"""

from insights.core.plugins import combiner
from insights.parsers.netstat import Netstat_I
from insights.parsers.ip import IpLinkInfo
from copy import deepcopy
HANDLD_FLAGS = ["LOWER_UP", "NOARP", "NO-CARRIER"]


@combiner([Netstat_I, IpLinkInfo])
class NetworkStats(object):
    """
    A combiner for working with both ``netstat -i`` and ``ip -s link`` That is if
    ``netstat -i`` gets deprecated then we can continue with ``ip -s link``.

    This interface closely follow interface Netstat_I it has following Attributes:

    Attributes:
        group_by_iface(dict): This property returns network stats organised per interface in dict format.
        data (list): List of network stats as per devices.
    """

    def __init__(self, net_stats, ip_stats):
        self.group_by_iface = {}
        self.data = []
        if net_stats:
            self.data = net_stats.data
            ldata = deepcopy(self.data)
            self.group_by_iface = self._group_by_iface(ldata)
        elif ip_stats:
            self.group_by_iface, self.data = self._organize_for_netstat(ip_stats.data)

    def _group_by_iface(self, ldata):
        periface = {}
        for ifdata in ldata:
            iface_name = ifdata["Iface"]
            del ifdata["Iface"]
            periface[iface_name] = ifdata
        return periface

    def _organize_for_netstat(self, ldata):

        def _get_flags(flags):
            flg_str = ""
            if "NOARP" in flags:
                flg_str += "O"
            elif "NO-CARRIER" not in flags or "LOWER_UP" in flags:
                flg_str += "R"
            for i in flags:
                if i not in HANDLD_FLAGS:
                    flg_str += str(i[0])
            flg_str = ''.join(sorted(flg_str))
            return flg_str

        stats = {}
        data = []
        map_table = {"MTU": "mtu", "RX-OK": "rx_packets", "RX-ERR": "rx_errors",
                "RX-DRP": "rx_dropped", "RX-OVR": "rx_overrun",
                "TX-OK": "tx_packets", "TX-ERR": "tx_errors", "Iface": "name",
                "TX-DRP": "tx_dropped"}

        for (k, v) in ldata.items():
            ifstat = {}
            for key_idx, value_idx in map_table.items():
                ifstat[key_idx] = v[value_idx]
            flg_str = _get_flags(v["flags"])
            ifstat["Flg"] = flg_str
            data_add = deepcopy(ifstat)
            data.append(data_add)
            del ifstat["Iface"]
            stats[v["name"]] = ifstat
        return stats, data



import pytest

from insights.components.openstack import IsOpenStackCompute
from insights.core.exceptions import SkipComponent
from insights.parsers.ps import PsAuxcww
from insights.tests import context_wrap

PS_MULTIPATHD = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START    TIME  COMMAND
root         9  0.0  0.0      0     0 ?        S    Mar24   0:05  multipathd
""".strip()


PS_OSP_COMPUTE = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START    TIME  COMMAND
nova     24928  0.1  1.1 390180 89368 ?        Ss   11:30   0:09  nova-compute
root         9  0.0  0.0      0     0 ?        S    Mar24   0:05  multipathd
""".strip()

PS_OSP_DIRECTOR = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START    TIME  COMMAND
nova     24928  0.1  1.1 390180 89368 ?        Ss   11:30   0:09  nova-compute
nova     24928  0.1  1.1 390180 89368 ?        Ss   11:30   0:09  nova-conductor
root         9  0.0  0.0      0     0 ?        S    Mar24   0:05  multipathd
""".strip()

PS_OSP_CONTROLLER = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START    TIME  COMMAND
nova     24928  0.1  1.1 390180 89368 ?        Ss   11:30   0:09  nova-conductor
root         9  0.0  0.0      0     0 ?        S    Mar24   0:05  multipathd
""".strip()


def test_generic_process():
    """The ``psauxcww`` does not have ``nova-compute`` process."""
    ps = PsAuxcww(context_wrap(PS_MULTIPATHD))
    with pytest.raises(SkipComponent) as e:
        IsOpenStackCompute(ps)
    assert "Not OpenStack Compute node" in str(e)


def test_compute_node():
    ps = PsAuxcww(context_wrap(PS_OSP_COMPUTE))
    result = IsOpenStackCompute(ps)
    assert isinstance(result, IsOpenStackCompute)


def test_controller_node():
    ps = PsAuxcww(context_wrap(PS_OSP_CONTROLLER))
    with pytest.raises(SkipComponent) as e:
        IsOpenStackCompute(ps)
    assert "Not OpenStack Compute node" in str(e)


def test_director_node():
    # A director act as an Compute as well as a Controller node.
    ps = PsAuxcww(context_wrap(PS_OSP_DIRECTOR))
    result = IsOpenStackCompute(ps)
    assert isinstance(result, IsOpenStackCompute)



"""
This module defines all datasources for JDR report
"""

from functools import partial
from insights.specs import Specs
from insights.core.plugins import datasource
from insights.core.context import JDRContext
from insights.core.spec_factory import simple_file, foreach_collect, first_file, glob_file, listdir

first_file = partial(first_file, context=JDRContext)
glob_file = partial(glob_file, context=JDRContext)
simple_file = partial(simple_file, context=JDRContext)
foreach_collect = partial(foreach_collect, context=JDRContext)


class JDRSpecs(Specs):
    """A class for all the JDR report datasources"""

    jboss_standalone_server_log = glob_file("JBOSS_HOME/standalone/log/server.log")

    @datasource(jboss_standalone_server_log, context=JDRContext, multi_output=True)
    def jboss_standalone_conf_file(broker):
        """Get which jboss standalone conf file is using from server log"""
        log_files = broker[JDRSpecs.jboss_standalone_server_log]
        if log_files:
            log_content = log_files[-1].content
            results = []
            for line in log_content:
                if "sun.java.command =" in line and ".jdr" not in line and "-Djboss.server.base.dir" in line:
                    results.append(line)
            if results:
                # default is standalone.xml
                config_xml = 'standalone.xml'
                java_command = results[-1]
                if '--server-config' in java_command:
                    config_xml = java_command.split('--server-config=')[1].split()[0]
                elif '-c ' in java_command:
                    config_xml = java_command.split('-c ')[1].split()[0]
                return [config_xml]
        return []

    # Get file content according the file name from server.log
    jboss_standalone_main_config = foreach_collect(jboss_standalone_conf_file, "JBOSS_HOME/standalone/configuration/%s")

    jboss_domain_servers = listdir("JBOSS_HOME/domain/servers/", context=JDRContext)
    jboss_domain_server_log = foreach_collect(jboss_domain_servers, "JBOSS_HOME/domain/servers/%s/log/server.log")



# -*- coding: UTF-8 -*-
import doctest

from insights.parsers import ls as ls_module
from insights.parsers.ls import FileListing, LSlan, LSlHFiles
from insights.tests import context_wrap

SINGLE_DIRECTORY = """
ls: cannot access ' _non_existing_': No such file or directory
/etc/pki/tls:
total 32
drwxr-xr-x.  5 root root  4096 Jun 28  2017 .
drwxr-xr-x. 15 root root  4096 Aug 10 09:42 ..
lrwxrwxrwx.  1 root root    49 Jun 28  2017 cert.pem -> /etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem
drwxr-xr-x.  2 root root  4096 Jun 28  2017 certs
drwxr-xr-x.  2 root root  4096 Mar 29  2017 misc
-rw-r--r--.  1 root root 10923 Feb  7  2017 openssl.cnf
drwxr-xr-x.  2 root root  4096 Feb  7  2017 private
"""

MULTIPLE_DIRECTORIES = """
/etc/sysconfig:
total 96
drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
-rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
-rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grubX

/etc/rc.d/rc3.d:
total 4
drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd
"""

COMPLICATED_FILES = """
/tmp:
total 16
dr-xr-xr-x.  2 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 10 0 0     4096 Mar  4 16:19 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 menu.lst -> ./grub.conf
brw-rw----.  1 0 6 253,  10 Aug  4 16:56 dm-10
crw-------.  1 0 0 10,  236 Jul 25 10:00 control
srw-------.  1 26214 17738 0 Oct 19 08:48 geany_socket.c46453c2
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 File name with spaces in it!
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 Unicode ÅÍÎÏÓÔÒÚÆ☃ madness.txt
drwxr-xr-x+  2 0 0       41 Jul  6 23:32 additional_ACLs
-rw-rw----.  1 0 6 253,  10 Aug  4 16:56 comma in size currently valid
brw-rw----.  1 0 6  1048576 Aug  4 16:56 block dev with no comma also valid
-rwxr-xr-x.  2 0 0     1024 Jul  6 23:32 file_name_ending_with_colon:
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 link with spaces -> ../file with spaces
"""

SELINUX_DIRECTORY = """
/boot:
total 3
-rw-r--r--. root root system_u:object_r:boot_t:s0      config-3.10.0-267
drwxr-xr-x. root root system_u:object_r:boot_t:s0      grub2
-rw-r--r--. root root system_u:object_r:boot_t:s0      initramfs-0-rescue
"""

FILES_CREATED_WITH_SELINUX_DISABLED = """
/dev/mapper:
total 2
lrwxrwxrwx 1 0 0 7 Apr 27 05:34 lv_cpwtk001_data01 -> ../dm-7
lrwxrwxrwx 1 0 0 7 Apr 27 05:34 lv_cpwtk001_redo01 -> ../dm-8
"""

BAD_DIRECTORY_ENTRIES = """
dr-xr-xr-x.  2 0 0     4096 Mar  4 16:19 dir entry with no dir header
total 3

/badness:
    -rwxr-xr-x. 0 0    1 Sep 12 2010 indented entry
xr-xr--r--. 0 0        1 Sep 12  2010 bad file type
-rxr-xr-x.  0 0        1 Sep 12  2010 missing user w permission
-rwxr-xr-x  0 0        1 Sep 12  2010 missing ACL dot
-rw-r--r--. user with spaces group 2 Oct 3 2011 user with spaces
-rw-r--r--. user group with spaces 2 Oct 3 2011 group with spaces
dr-xr-xr-x. -42 -63 1271 Jan  6  2008 Negative user and group numbers
dr-xr-xr-x. 1 7 123, 124, 125 Jan 6 2008 Three comma blocks in size
brw-rw----. 1 0 6 123456 Aug 4 16:56 two size blocks
prw-rw----. 1000 1000  0  6 2007 Month missing
prw-rw----. 1000 1000  0 No 6 2007 Month too short
prw-rw----. 1000 1000  0 November 6 2007 Month too long
prw-rw----. 1000 1000  0 Nov  2007 Day too long
prw-rw----. 1000 1000  0 Nov 126 2007 Day too long
prw-rw----. 1000 1000  0 Nov 126  Year missing
prw-rw----. 1000 1000  0 Nov 126 20107 Year too long
prw-rw----. 1000 1000  0 Nov 12 :56 Missing hour
prw-rw----. 1000 1000  0 Nov 12 723:56 Hour too long
prw-rw----. 1000 1000  0 Nov 12 23: Missing minute
prw-rw----. 1000 1000  0 Nov 12 23:3 Minute too short
prw-rw----. 1000 1000  0 Nov 12 23:357 Minute too long
-rw------ 1 root root 762 Sep 23 002 permission too short
bash: ls: command not found
-rw------ 1 root root 762 Se
-rw------- 1 ro:t root 762 Sep 23 002 colon in uid
-rw------- 1 root r:ot 762 Sep 23 002 colon in gid
-rwasdfas- 1 root root 762 Sep 23 002 bad permissions block
-rwx/----- 1 root root 762 Sep 23 002 slash in permissions block
-rwx------ 1 root root 762 Sep 23 002 /slashes/in/filename
/rwasdfas- 1 root root 762 Sep 23 002 slash in file type and no colon on end
/usr/bin/ls: cannot access /boot/grub2/grub.cfg: No such file or directory
cannot access /boot/grub2/grub.cfg: No such file or directory
No such file or directory
adsf
"""

FILE_LISTING_DOC = '''
        /boot:
        total 187380
        dr-xr-xr-x.  3 0 0   4096 Mar  4 16:19 .
        dr-xr-xr-x. 19 0 0   4096 Jul 14 09:10 ..
        -rw-r--r--.  1 0 0 123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64

        /etc/sysconfig:
        total 96
        drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
        drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
        drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
        drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
        -rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
        -rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
        lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub
'''

LS_FILE_PERMISSIONS_DOC = '''
/bin/ls: cannot access /boot/grub/grub.conf: No such file or directory
/bin/ls: cannot access fstab_mounted.devices: No such file or directory
/bin/ls: cannot access pvs.devices: No such file or directory
-rw-r--r--. 1 root  root      46 Apr 24  2024 /etc/redhat-release
-rw-r--r--. 1 liuxc wheel 664118 Feb 20 14:40 /var/log/messages
brw-rw----. 1 root  disk 252, 1 May 16 01:30 /dev/vda1
'''
# Note - should we test for anomalous but parseable entries?  E.g. block
# devices without a major/minor number?  Or non-devices that have a comma in
# the size?  Permissions that don't make sense?  Dates that don't make sense
# but still fit the patterns?  What should the parser do with such entries?


def test_single_directory():
    # ctx = context_wrap(SINGLE_DIRECTORY, path='ls_-la_.etc.pki.tls')
    ctx = context_wrap(SINGLE_DIRECTORY)
    dirs = FileListing(ctx)

    assert '/etc/pki/tls' in dirs
    assert '/etc/pki/tls/certs' not in dirs
    assert '/etc/pki' not in dirs
    assert '/etc' not in dirs
    assert dirs['/etc/pki/tls']['name'] == '/etc/pki/tls'

    assert dirs.files_of('/etc/pki/tls') == ['cert.pem', 'openssl.cnf']
    assert dirs.dirs_of('/etc/pki/tls') == ['.', '..', 'certs', 'misc', 'private']
    assert dirs.total_of('/etc/pki/tls') == 32
    assert dirs.files_of('non-exist') == []
    assert dirs.dirs_of('non-exist') == []
    assert dirs.total_of('non-exist') == 0


def test_multiple_directories():
    dirs = FileListing(context_wrap(MULTIPLE_DIRECTORIES, path='ls_-la_.etc'))

    assert '/etc/sysconfig' in dirs
    assert '/etc/rc.d/rc3.d' in dirs
    assert '/etc/rc.d/rc4.d' not in dirs

    esc = dirs['/etc/sysconfig']
    assert sorted(esc.keys()) == sorted(['entries', 'files', 'dirs', 'specials', 'total', 'name'])

    assert dirs.files_of('/etc/sysconfig') == ['ebtables-config', 'firewalld', 'grub', 'grubX']
    assert dirs.dirs_of('/etc/sysconfig') == ['.', '..', 'cbq', 'console']
    assert dirs.specials_of('/etc/sysconfig') == []
    assert dirs.files_of('non-exist') == []
    assert dirs.dirs_of('non-exist') == []
    assert dirs.specials_of('non-exist') == []
    assert dirs.listing_of('non-exist') == {}
    assert dirs.total_of('non-exist') == 0
    assert dirs.dir_contains('non-exist', 'test') is False
    assert dirs.dir_entry('non-exist', 'test') == {}

    # Testing the main features
    listing = dirs.listing_of('/etc/sysconfig')
    assert listing['..'] == {
        'type': 'd',
        'perms': 'rwxr-xr-x.',
        'links': 77,
        'owner': '0',
        'group': '0',
        'size': 8192,
        'date': 'Jul 13 03:55',
        'name': '..',
        'dir': '/etc/sysconfig',
    }
    assert listing['cbq'] == {
        'type': 'd',
        'perms': 'rwxr-xr-x.',
        'links': 2,
        'owner': '0',
        'group': '0',
        'size': 41,
        'date': 'Jul  6 23:32',
        'name': 'cbq',
        'dir': '/etc/sysconfig',
    }
    assert listing['firewalld'] == {
        'type': '-',
        'perms': 'rw-r--r--.',
        'links': 1,
        'owner': '0',
        'group': '0',
        'size': 72,
        'date': 'Sep 15  2015',
        'name': 'firewalld',
        'dir': '/etc/sysconfig',
    }
    assert listing['grub'] == {
        'type': 'l',
        'perms': 'rwxrwxrwx.',
        'links': 1,
        'owner': '0',
        'group': '0',
        'size': 17,
        'date': 'Jul  6 23:32',
        'name': 'grub',
        'link': '/etc/default/grub',
        'dir': '/etc/sysconfig',
    }

    listing = dirs.listing_of('/etc/rc.d/rc3.d')
    assert listing['..'] == {
        'type': 'd',
        'perms': 'rwxr-xr-x.',
        'links': 10,
        'owner': '0',
        'group': '0',
        'size': 4096,
        'date': 'Sep 16  2015',
        'name': '..',
        'dir': '/etc/rc.d/rc3.d',
    }
    assert listing['K50netconsole'] == {
        'type': 'l',
        'perms': 'rwxrwxrwx.',
        'links': 1,
        'owner': '0',
        'group': '0',
        'size': 20,
        'date': 'Jul  6 23:32',
        'name': 'K50netconsole',
        'link': '../init.d/netconsole',
        'dir': '/etc/rc.d/rc3.d',
    }

    assert dirs.total_of('/etc/sysconfig') == 96
    assert dirs.total_of('/etc/rc.d/rc3.d') == 4

    assert dirs.dir_contains('/etc/sysconfig', 'firewalld')
    assert dirs.dir_entry('/etc/sysconfig', 'grub') == {
        'type': 'l',
        'perms': 'rwxrwxrwx.',
        'links': 1,
        'owner': '0',
        'group': '0',
        'size': 17,
        'date': 'Jul  6 23:32',
        'name': 'grub',
        'link': '/etc/default/grub',
        'dir': '/etc/sysconfig',
    }

    assert dirs.path_entry('/etc/sysconfig/cbq') == {
        'type': 'd',
        'perms': 'rwxr-xr-x.',
        'links': 2,
        'owner': '0',
        'group': '0',
        'size': 41,
        'date': 'Jul  6 23:32',
        'name': 'cbq',
        'dir': '/etc/sysconfig',
    }
    assert dirs.path_entry('no_slash') is None
    assert dirs.path_entry('/') is None
    assert dirs.path_entry('/foo') is None
    assert dirs.path_entry('/etc/sysconfig/notfound') is None


def test_raw_entry_of_permissions_of():
    dirs = FileListing(context_wrap(MULTIPLE_DIRECTORIES, path='ls_-la_.etc'))

    raw_entry = 'drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..'
    assert dirs.raw_entry_of('/etc/sysconfig', '..') == raw_entry
    assert dirs.permissions_of('/etc/sysconfig', '..').line == raw_entry

    raw_entry = 'drwxr-xr-x. 2 0 0 41 Jul  6 23:32 cbq'
    assert dirs.raw_entry_of('/etc/sysconfig', 'cbq') == raw_entry
    assert dirs.permissions_of('/etc/sysconfig', 'cbq').line == raw_entry

    raw_entry = '-rw-r--r--. 1 0 0 72 Sep 15  2015 firewalld'
    assert dirs.raw_entry_of('/etc/sysconfig', 'firewalld') == raw_entry
    assert dirs.permissions_of('/etc/sysconfig', 'firewalld').line == raw_entry

    raw_entry = 'lrwxrwxrwx. 1 0 0 17 Jul  6 23:32 grub -> /etc/default/grub'
    assert dirs.raw_entry_of('/etc/sysconfig', 'grub') == raw_entry
    assert dirs.permissions_of('/etc/sysconfig', 'grub').line == raw_entry
    # abnormal case, no '->' for links
    raw_entry = 'lrwxrwxrwx. 1 0 0 17 Jul  6 23:32 grubX'
    assert dirs.raw_entry_of('/etc/sysconfig', 'grubX') == raw_entry
    assert dirs.permissions_of('/etc/sysconfig', 'grubX').line == raw_entry
    # no such target file
    dirs.raw_entry_of('/etc/sysconfig', 'test') is None
    dirs.permissions_of('/etc/sysconfig', 'test') is None
    # no such dir
    dirs.raw_entry_of('/etc/test', 'grub') is None
    dirs.permissions_of('/etc/test', 'grub') is None

    # New input
    dirs = FileListing(context_wrap(COMPLICATED_FILES))

    raw_entry = 'brw-rw----. 1 0 6 253, 10 Aug  4 16:56 dm-10'
    assert dirs.raw_entry_of('/tmp', 'dm-10') == raw_entry
    assert dirs.permissions_of('/tmp', 'dm-10').line == raw_entry

    # New input
    dirs = FileListing(context_wrap(SELINUX_DIRECTORY))

    raw_entry = 'drwxr-xr-x. root root system_u:object_r:boot_t:s0 grub2'
    assert dirs.raw_entry_of('/boot', 'grub2') == raw_entry
    # no permissions_of test

    # New input
    dirs = FileListing(context_wrap(FILES_CREATED_WITH_SELINUX_DISABLED))
    raw_entry = 'lrwxrwxrwx 1 0 0 7 Apr 27 05:34 lv_cpwtk001_data01 -> ../dm-7'
    assert dirs.raw_entry_of('/dev/mapper', 'lv_cpwtk001_data01') == raw_entry
    assert dirs.permissions_of('/dev/mapper', 'lv_cpwtk001_data01').line == raw_entry


def test_complicated_directory():
    dirs = FileListing(context_wrap(COMPLICATED_FILES))

    # Test the things we expect to be different:
    listing = dirs.listing_of('/tmp')
    assert listing['config-3.10.0-229.14.1.el7.x86_64']['type'] == '-'
    assert listing['menu.lst']['type'] == 'l'
    assert listing['menu.lst']['link'] == './grub.conf'
    assert dirs.dir_entry('/tmp', 'dm-10') == {
        'type': 'b',
        'perms': 'rw-rw----.',
        'links': 1,
        'owner': '0',
        'group': '6',
        'major': 253,
        'minor': 10,
        'date': 'Aug  4 16:56',
        'name': 'dm-10',
        'dir': '/tmp',
    }
    assert listing['dm-10']['type'] == 'b'
    assert listing['dm-10']['major'] == 253
    assert listing['dm-10']['minor'] == 10
    assert listing['control']['type'] == 'c'
    assert listing['control']['major'] == 10
    assert listing['control']['minor'] == 236
    assert listing['geany_socket.c46453c2']['type'] == 's'
    assert listing['geany_socket.c46453c2']['size'] == 0
    assert listing['link with spaces']['type'] == 'l'
    assert listing['link with spaces']['link'] == '../file with spaces'

    # Check that things that _shouldn't_ be there _aren't_
    assert 'size' not in listing['dm-10']
    assert 'size' not in listing['control']

    # Tricky file names
    assert 'File name with spaces in it!' in listing
    assert 'Unicode ÅÍÎÏÓÔÒÚÆ☃ madness.txt' in listing
    assert 'file_name_ending_with_colon:' in listing
    assert dirs.dir_contains('/tmp', 'File name with spaces in it!')
    assert dirs.dir_contains('/tmp', 'Unicode ÅÍÎÏÓÔÒÚÆ☃ madness.txt')
    assert dirs.dir_contains('/tmp', 'file_name_ending_with_colon:')

    # For devices missing a comma in their 'size', size is also unconverted
    assert 'block dev with no comma also valid' in listing
    assert listing['block dev with no comma also valid']['size'] == 1048576
    assert 'major' not in listing['block dev with no comma also valid']
    assert 'minor' not in listing['block dev with no comma also valid']

    # Extended ACLs
    assert 'additional_ACLs' in listing
    assert listing['additional_ACLs']['perms'] == 'rwxr-xr-x+'


def test_selinux_directory():
    dirs = FileListing(context_wrap(SELINUX_DIRECTORY))

    # Test that one entry is exactly what we expect it to be.
    expected = {
        'type': 'd',
        'perms': 'rwxr-xr-x.',
        'owner': 'root',
        'group': 'root',
        'se_user': 'system_u',
        'se_role': 'object_r',
        'se_type': 'boot_t',
        'se_mls': 's0',
        'name': 'grub2',
        'dir': '/boot',
    }
    actual = dirs.dir_entry('/boot', 'grub2')
    assert actual == expected


def test_files_created_with_selinux_disabled():
    dirs = FileListing(context_wrap(FILES_CREATED_WITH_SELINUX_DISABLED))

    # Test that one entry is exactly what we expect it to be.
    assert dirs.dir_entry('/dev/mapper', 'lv_cpwtk001_data01') == {
        'group': '0',
        'name': 'lv_cpwtk001_data01',
        'links': 1,
        'perms': 'rwxrwxrwx',
        'owner': '0',
        'link': '../dm-7',
        'date': 'Apr 27 05:34',
        'type': 'l',
        'dir': '/dev/mapper',
        'size': 7,
    }


def test_doc_example():
    env = {
        'ls_lan': LSlan(context_wrap(FILE_LISTING_DOC)),
        'ls_files': LSlHFiles(context_wrap(LS_FILE_PERMISSIONS_DOC)),
    }
    failed, total = doctest.testmod(ls_module, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import bond
from insights.parsers.bond import Bond
from insights.tests import context_wrap

CONTEXT_PATH = "proc/net/bonding/bond0"
CONTEXT_PATH_1 = "proc/net/bonding/test0"

BONDINFO_1 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: load balancing (round-robin)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eno1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f8
Slave queue ID: 0

Slave Interface: eno2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f9
Slave queue ID: 0
""".strip()

BONDINFO_MODE_4 = """
Ethernet Channel Bonding Driver: v3.2.4 (January 28, 2008)

Bonding Mode: IEEE 802.3ad Dynamic link aggregation
Transmit Hash Policy: layer2 (0)
MII Status: up
MII Polling Interval (ms): 500
Up Delay (ms): 0
Down Delay (ms): 0

802.3ad info
LACP rate: slow
Active Aggregator Info:
        Aggregator ID: 3
        Number of ports: 1
        Actor Key: 17
        Partner Key: 1
        Partner Mac Address: 00:00:00:00:00:00

Slave Interface: eth1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:42:fc
Aggregator ID: 3

Slave Interface: eth2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Speed
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:02:7e
Aggregator ID: 2
""".strip()

BONDINFO_MODE_2 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: load balancing (xor)
Transmit Hash Policy: layer2+3 (2)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eno1
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f8
Slave queue ID: 0

Slave Interface: eno2
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 2c:44:fd:80:5c:f9
Slave queue ID: 0
""".strip()


BONDINFO_CORRUPT = """
Link Failure Count: 0
Permanent HW addr: 00:16:35:5e:02:7e
Aggregator ID:
""".strip()

BONDINFO_UNKNOWN_BOND_MODE = """
Bonding Mode: reverse proximity hash combination mode
""".strip()


BONDINFO_MODE_5 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: fault-tolerance (active-backup)
Primary Slave: None
Currently Active Slave: enp17s0f0
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: enp17s0f0
MII Status: up
Speed: 10000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:1f:f3:af:d3:f0
Slave queue ID: 0

Slave Interface: enp17s0f1
MII Status: up
Speed: 10000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 00:1f:f3:af:d3:f1
Slave queue ID: 0
""".strip()


BONDINFO_MODE_6 = BONDINFO_MODE_5.replace("Currently Active Slave: enp17s0f0", "")

BONDINFO_MODE_7 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: fault-tolerance (active-backup)
Primary Slave: em3 (primary_reselect failure)
Currently Active Slave: em3
MII Status: up
MII Polling Interval (ms): 0
Up Delay (ms): 0
Down Delay (ms): 0
ARP Polling Interval (ms): 1000
ARP IP target/s (n.n.n.n form): 10.152.1.1

Slave Interface: em3
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 92028
Permanent HW addr: 00:1f:f3:af:d3:f1
Slave queue ID: 0

Slave Interface: p2p3
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 71524
Permanent HW addr: 00:1f:f3:af:d3:f1
Slave queue ID: 0
""".strip()

BOND_MODE_4 = """
Ethernet Channel Bonding Driver: v3.7.1 (April 27, 2011)

Bonding Mode: IEEE 802.3ad Dynamic link aggregation
Transmit Hash Policy: layer2 (0)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 2000
Down Delay (ms): 1000

802.3ad info
LACP rate: slow
Min links: 0
Aggregator selection policy (ad_select): stable
System priority: 65535
System MAC address: 08:00:27:99:a3:6b
Active Aggregator Info:
\t\t\t\tAggregator ID: 1
\t\t\t\tNumber of ports: 1
\t\t\t\tActor Key: 9
\t\t\t\tPartner Key: 1
\t\t\t\tPartner Mac Address: 00:00:00:00:00:00

Slave Interface: enp0s9
MII Status: up
Speed: 1000 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 08:00:27:99:a3:6b
Slave queue ID: 0
Aggregator ID: 1
Actor Churn State: none
Partner Churn State: churned
Actor Churned Count: 0
Partner Churned Count: 1
details actor lacp pdu:
    system priority: 65535
    system mac address: 08:00:27:99:a3:6b
    port key: 9
    port priority: 255
    port number: 1
    port state: 77
details partner lacp pdu:
    system priority: 65535
    system mac address: 00:00:00:00:00:00
    oper key: 1
    port priority: 255
    port number: 1
    port state: 1

Slave Interface: enp0s8
MII Status: down
Speed: Unknown
Duplex: Unknown
Link Failure Count: 0
Permanent HW addr: 08:00:27:a2:8d:f5
Slave queue ID: 0
Aggregator ID: 2
Actor Churn State: churned
Partner Churn State: churned
Actor Churned Count: 1
Partner Churned Count: 1
details actor lacp pdu:
    system priority: 65535
    system mac address: 08:00:27:99:a3:6b
    port key: 0
    port priority: 255
    port number: 2
    port state: 69
details partner lacp pdu:
    system priority: 65535
    system mac address: 00:00:00:00:00:00
    oper key: 1
    port priority: 255
    port number: 1
    port state: 1
""".strip()


def test_netstat_doc_examples():
    env = {
        'bond_info': Bond(context_wrap(BONDINFO_MODE_4)),
    }
    failed, total = doctest.testmod(bond, globs=env)
    assert failed == 0


def test_bond_class():
    bond_obj = Bond(context_wrap(BONDINFO_1, CONTEXT_PATH))
    assert bond_obj.file_name == 'bond0'
    assert not bond_obj.partner_mac_address
    assert bond_obj.bond_mode == '0'
    assert bond_obj.slave_interface == ['eno1', 'eno2']
    assert bond_obj.up_delay == '0'
    assert bond_obj.down_delay == '0'
    assert bond_obj.data['eno1']['speed'] == '1000 Mbps'
    assert bond_obj.data['eno1']['mii_status'] == 'up'
    assert bond_obj.data['eno2']['mii_status'] == 'up'

    bond_obj = Bond(context_wrap(BONDINFO_MODE_4, CONTEXT_PATH))
    assert bond_obj.bond_mode == '4'
    assert bond_obj.partner_mac_address == "00:00:00:00:00:00"
    assert bond_obj.aggregator_id == ['3', '3', '2']
    assert bond_obj.xmit_hash_policy == "layer2"
    assert bond_obj.active_slave is None

    bond_obj = Bond(context_wrap(BONDINFO_CORRUPT, CONTEXT_PATH))
    assert bond_obj.bond_mode is None
    assert bond_obj.slave_interface == []
    assert not bond_obj.xmit_hash_policy

    bond_obj = Bond(context_wrap(BONDINFO_MODE_2, CONTEXT_PATH))
    assert bond_obj.xmit_hash_policy == "layer2+3"

    bond_obj = Bond(context_wrap(BONDINFO_MODE_5, CONTEXT_PATH))
    assert bond_obj.bond_mode == '1'
    assert bond_obj.active_slave == "enp17s0f0"

    bond_obj_2 = Bond(context_wrap(BONDINFO_MODE_6, CONTEXT_PATH))
    assert bond_obj_2.bond_mode == '1'
    assert bond_obj_2.active_slave is None

    bond_obj_3 = Bond(context_wrap(BONDINFO_1, CONTEXT_PATH))
    assert bond_obj_3.file_name == 'bond0'
    assert bond_obj_3.slave_interface == ['eno1', 'eno2']
    assert bond_obj_3.slave_duplex == ['full', 'full']
    assert bond_obj_3.slave_speed == ['1000 Mbps', '1000 Mbps']
    assert bond_obj_3.slave_link_failure_count == ['0', '0']
    assert bond_obj_3.mii_status == ['up', 'up', 'up']
    assert bond_obj_3.arp_polling_interval is None
    assert bond_obj_3.arp_ip_target is None
    assert bond_obj_3.mii_polling_interval == "100"

    bond_obj_4 = Bond(context_wrap(BONDINFO_MODE_7, CONTEXT_PATH))
    assert bond_obj_4.file_name == 'bond0'
    assert bond_obj_4.arp_polling_interval == "1000"
    assert bond_obj_4.arp_ip_target == "10.152.1.1"
    assert bond_obj_4.mii_polling_interval == "0"
    assert bond_obj_4.primary_slave == 'em3 (primary_reselect failure)'

    bond_obj = Bond(context_wrap(BOND_MODE_4, CONTEXT_PATH))
    assert bond_obj.file_name == 'bond0'
    assert bond_obj.up_delay == '2000'
    assert bond_obj.down_delay == '1000'
    assert bond_obj.data['mii_status'] == 'up'
    assert bond_obj.data['enp0s9']['mii_status'] == 'up'
    assert bond_obj.data['enp0s8']['mii_status'] == 'down'
    assert bond_obj.data['enp0s8']['aggregator_id'] == '2'
    assert bond_obj.data['enp0s9']['aggregator_id'] == '1'
    assert bond_obj.data['enp0s8']['permanent_hw_addr'] == '08:00:27:a2:8d:f5'
    assert bond_obj.data['enp0s9']['permanent_hw_addr'] == '08:00:27:99:a3:6b'

    with pytest.raises(ParseException) as exc:
        bond_obj = Bond(context_wrap(BONDINFO_UNKNOWN_BOND_MODE, CONTEXT_PATH))
        assert not bond_obj.bond_mode
    assert 'Unrecognised bonding mode' in str(exc)


def test_bond_class_different_file_name():
    bond_obj = Bond(context_wrap(BONDINFO_1, CONTEXT_PATH_1))
    assert bond_obj.file_name == 'test0'
    assert not bond_obj.partner_mac_address
    assert bond_obj.bond_mode == '0'
    assert bond_obj.slave_interface == ['eno1', 'eno2']
    assert bond_obj.up_delay == '0'
    assert bond_obj.down_delay == '0'
    assert bond_obj.data['eno1']['speed'] == '1000 Mbps'
    assert bond_obj.data['eno1']['mii_status'] == 'up'
    assert bond_obj.data['eno2']['mii_status'] == 'up'



import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers.crypto_policies import CryptoPoliciesConfig
from insights.tests import context_wrap

CONFIG = """
DEFAULT
""".strip()

CONFIG_COMMENTED = """
# This file should contain a single keyword, the crypto policy to
# be applied by default to applications. The available policies are 
# restricted to the following profiles.
#
# * LEGACY: Ensures maximum compatibility with legacy systems (64-bit
#   security)
#
# * DEFAULT: A reasonable default for today's standards (112-bit security).
#
# * FUTURE: A level that will provide security on a conservative level that is
#   believed to withstand any near-term future attacks (128-bit security).
#
# * FIPS: Policy that enables only FIPS 140-2 approved or allowed algorithms.
#
# After modifying this file, you need to run update-crypto-policies
# for the changes to propagate.
#
DEFAULT
""".strip()  # noqa


def test_crypto_policies_config():
    result = CryptoPoliciesConfig(context_wrap(CONFIG))
    assert result.value == "DEFAULT"


def test_crypto_policies_commented():
    result = CryptoPoliciesConfig(context_wrap(CONFIG_COMMENTED))
    assert result.value == "DEFAULT"


def test_crypto_policies_config_empty():
    with pytest.raises(SkipComponent):
        CryptoPoliciesConfig(context_wrap(""))



"""
1. The rule uses one Combiner that depends upon a filterable Parser.
2. The rule does also depend upon (import) the filterable Parser.
3. The filterable Parser is NOT filtered in the rule.

=> Exception is expected
"""
from insights import rule, make_pass
from insights.combiners.os_release import OSRelease
from insights.parsers.dmesg import DmesgLineList


@rule(OSRelease, DmesgLineList)
def parser_is_not_filtered(osr, dmesg):
    return make_pass("FAKE RESULT")



from insights.parsers.dmesg import DmesgLineList
from insights.tests import context_wrap

MSGINFO = """
lo: Dropping TSO features since no CSUM feature.
bnx2 0000:0b:00.0: irq 86 for MSI/MSI-X
bnx2 0000:0b:00.0: eth0: using MSIX
(null): Dropping TSO features since no CSUM feature.
(null): Dropping TSO6 features since no CSUM feature.
bonding: bond0: enslaving eth0 as a backup interface with a down link.
HP HPSA Driver (v 3.4.4-1-RH2)
  alloc irq_desc for 34 on node 0
  alloc kstat_irqs on node 0
scsi4 : hpsa
hpsa 0000:02:00.0: RAID              device c4b3t0l0 added.
hpsa 0000:02:00.0: Direct-Access     device c4b0t0l0 added.
scsi 4:3:0:0: RAID              HP       P420i            3.04 PQ: 0 ANSI: 5
scsi 4:0:0:0: Direct-Access     HP       LOGICAL VOLUME   3.04 PQ: 0 ANSI: 5
HP HPSA Driver (v 3.4.4-1-RH2)
[    8.687252] HP HPSA Driver (v 3.4.4-1-RH2) 2.5.0
lo: Dropping TSO features since no CSUM feature duplicated.
""".strip()

TS_MSGINFO = """
[    0.000000] tsc: Fast TSC calibration using PIT
[    0.000000] tsc: Detected 2693.827 MHz processor
[    0.000026] Calibrating delay loop (skipped), value calculated using timer frequency.. 5387.65 BogoMIPS (lpj=2693827)
[    0.000028] pid_max: default: 32768 minimum: 301
[    0.000048] Security Framework initialized
[    0.000053] SELinux:  Initializing.
[    0.000059] SELinux:  Starting in permissive mode
[    0.001043] Dentry cache hash table entries: 2097152 (order: 12, 16777216 bytes)
[    0.003709] Inode-cache hash table entries: 1048576 (order: 11, 8388608 bytes)
[    0.004791] Mount-cache hash table entries: 4096
[    0.004944] Initializing cgroup subsys memory
[    0.004958] Initializing cgroup subsys net_prio
[    0.004982] CPU: Physical Processor ID: 0
[    0.004983] CPU: Processor Core ID: 0
[    0.004987] ENERGY_PERF_BIAS: Set to 'normal', was 'performance'
[    0.004987] ENERGY_PERF_BIAS: View and update with x86_energy_perf_policy(8)
[    0.005828] mce: CPU supports 9 MCE banks
[    0.005841] CPU0: Thermal monitoring enabled (TM1)
[    0.005851] Last level iTLB entries: 4KB 0, 2MB 0, 4MB 0
[    0.005852] Last level dTLB entries: 4KB 64, 2MB 0, 4MB 0
[    0.005853] tlb_flushall_shift: 6
[    0.005933] Freeing SMP alternatives: 28k freed
[    0.006814] ACPI: Core revision 20130517
[    0.014594] ACPI: All ACPI Tables successfully acquired
[    0.015670] ftrace: allocating 25815 entries in 101 pages
[    0.024837] smpboot: Max logical packages: 2
[    0.024847] DMAR: Host address width 39
[    0.025063] Enabling x2apic
[    0.025064] Enabled x2apic
[    0.025069] Switched APIC routing to cluster x2apic.
[    0.025482] ..TIMER: vector=0x30 apic1=0 pin1=2 apic2=-1 pin2=-1
[    0.035484] smpboot: CPU0: Intel(R) Core(TM) i7-4800MQ CPU @ 2.70GHz (fam: 06, model: 3c, stepping: 03)
[    0.035490] TSC deadline timer enabled

[10424.756954] perf: interrupt took too long (2507 > 2500), lowering kernel.perf_event_max_sample_rate to 79750
"""


def test_dmesg():
    dmesg_info = DmesgLineList(context_wrap(MSGINFO))
    assert sorted([i['raw_message'] for i in dmesg_info.get("HPSA")]) == sorted(
            ['HP HPSA Driver (v 3.4.4-1-RH2)', 'HP HPSA Driver (v 3.4.4-1-RH2)', '[    8.687252] HP HPSA Driver (v 3.4.4-1-RH2) 2.5.0'])
    assert len(dmesg_info.get("lo:")) == 2
    assert "Dropping" in dmesg_info
    assert dmesg_info.has_startswith("bonding:")
    assert not dmesg_info.has_startswith("xfs:")
    assert len(list(dmesg_info.get_after(0.0001))) == 2
    assert list(dmesg_info.get_after(0.0001))[0]['raw_message'] == '[    8.687252] HP HPSA Driver (v 3.4.4-1-RH2) 2.5.0'
    assert dmesg_info.logs_startwith('(null)') == ['(null): Dropping TSO features since no CSUM feature.', '(null): Dropping TSO6 features since no CSUM feature.']

    ts_info = DmesgLineList(context_wrap(TS_MSGINFO))
    assert ts_info
    assert ts_info.get('Processor Core ID')[0]['raw_message'] == '[    0.004983] CPU: Processor Core ID: 0'
    assert ts_info.has_startswith('ENERGY_PERF_BIAS')
    assert not ts_info.has_startswith('Intel(R) Core(TM)')
    assert 'tlb_flushall_shift' in ts_info
    assert len(list(ts_info.get_after(0.024847))) == 9
    assert len(list(ts_info.get_after(0.024847, 'x2apic'))) == 3
    assert ts_info.logs_startwith('perf') == ['perf: interrupt took too long (2507 > 2500), lowering kernel.perf_event_max_sample_rate to 79750']
    assert ts_info.logs_startwith('systemd') == []



import json

try:
    from unittest.mock import patch, mock_open
    builtin_open = "builtins.open"
except Exception:
    from mock import patch
    from mock.mock import mock_open
    builtin_open = "__builtin__.open"

from insights.client.connection import InsightsConnection
from insights.util.hostname import determine_hostname
from insights.client.config import InsightsConfig


class MockSession(object):
    def __init__(self):
        self.status_code = None
        self.text = None
        self.content = '{"display_name": "test"}'
        self.headers = dict()


def mock_init_session(obj):
    return MockSession()


@patch("insights.client.connection.InsightsConnection._legacy_upload_archive", return_value=None)
def test_cfacts_no_cleaning_1(legacy_upload):
    # 1. No need to cleaning as legacy_upload=True
    config = InsightsConfig(legacy_upload=True, obfuscate=True, obfuscate_hostname=True)
    conn = InsightsConnection(config)
    conn.upload_archive('data_collected', 'content_type')
    legacy_upload.assert_called()


@patch('insights.client.connection.InsightsConnection._init_session', mock_init_session)
@patch('insights.client.connection.InsightsConnection.post', return_value=MockSession())
@patch(builtin_open, new_callable=mock_open, read_data='')
@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.get_canonical_facts", return_value={'hostname': determine_hostname()})
@patch("insights.client.connection.logger")
def test_cfacts_no_cleaning_2(logger, facts, rm_conf, _open, post):
    # 2. No cleaning per client configuration
    config = InsightsConfig(legacy_upload=False, obfuscate=False)
    conn = InsightsConnection(config)
    conn.upload_archive('data_collected', 'content_type')
    cfacts = facts.return_value
    cfacts.update({"branch_info": {"remote_branch": -1, "remote_leaf": -1}, "satellite_id": -1})
    c_facts = json.dumps(cfacts)

    for log_message in logger.debug.mock_calls:
        if c_facts in str(log_message):
            break
    else:
        assert False, "Canonical facts weren't obfuscated or logged properly"


@patch('insights.client.connection.InsightsConnection._init_session', mock_init_session)
@patch('insights.client.connection.InsightsConnection.post', return_value=MockSession())
@patch(builtin_open, new_callable=mock_open, read_data='')
@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.get_canonical_facts", return_value={'hostname': determine_hostname(), 'ip': '10.0.0.1'})
@patch("insights.client.connection.logger")
def test_cfacts_cleaned(logger, facts, rm_conf, _open, post):
    # 3. Cleaned per client configuration
    config = InsightsConfig(legacy_upload=False, obfuscate=True, obfuscate_hostname=True)
    conn = InsightsConnection(config)
    conn.upload_archive('data_collected', 'content_type')

    for log_message in logger.debug.mock_calls:
        log_message = str(log_message)
        if (
            "hostname" in log_message
            and determine_hostname() not in log_message
            and "ip" in log_message
            and "10.0.0.1" not in log_message
        ):
            break
    else:
        assert False, "Canonical facts weren't obfuscated or logged properly"



from insights.cleaner import Cleaner
from insights.client.config import InsightsConfig


def test_obfuscate_hostname_and_ip():
    hostname = 'test1.abc.com'
    line = "test1.abc.com, 10.0.0.1 test1.abc.loc, 20.1.4.7 smtp.abc.com, 10.1.2.7 lite.abc.com"
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    result = pp.clean_content(line)
    assert 'example.com' in result
    assert '10.230.230' in result
    for item in line.split():
        assert item not in result


def test_clean_content_keyword_with_hostname_and_ip():
    hostname = 'test1.abc.com'
    line = "test1.abc.com, 10.0.0.1, test1.abc.loc, 20.1.4.7, smtp.abc.com, what's your name?, what day is today?"
    conf = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(conf, {'keywords': ['name', 'day']}, hostname)
    result = pp.clean_content(line)
    assert 'test1.abc.com' not in result
    assert '10.0.0.1' not in result
    assert '20.1.4.7' not in result
    assert 'name' not in result
    assert 'day' not in result
    assert 'keyword0' in result
    assert 'keyword1' in result



import doctest
import pytest

from insights.core.exceptions import ContentException, SkipComponent, ParseException
from insights.parsers import ansible
from insights.tests import context_wrap

AT_INPUT_NG_CMD = """
no such file or directory: /usr/share/ansible/telemetry/telemetry.py
"""

AT_INPUT_NG_CNT = """
{"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":13}},"version":"*"}},"ansible_core":{"version":"2.18.9rc1"}}

{"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":14}},"version":"*"}},"ansible_core":{"version":"2.19.9"}}]
"""

AT_INPUT = """
{"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":13}},"version":"*"}},"ansible_core":{"version":"2.18.9rc1"}}
{"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":14}},"version":"*"}},"ansible_core":{"version":"2.19.9"}}
""".strip()


def test_ansible_telemetry():
    ret = ansible.AnsibleTelemetry(context_wrap(AT_INPUT))
    assert len(ret) == 2
    assert ret[0]['ansible_core']['version'] == '2.18.9rc1'
    assert ret[1]['collections']['ansible.builtin']['resources']['action'] == {
        'ansible.builtin.command': 14
    }

    with pytest.raises(SkipComponent):
        ansible.AnsibleTelemetry(context_wrap(""))

    with pytest.raises(ContentException):
        ansible.AnsibleTelemetry(context_wrap(AT_INPUT_NG_CMD))

    with pytest.raises(ParseException):
        ansible.AnsibleTelemetry(context_wrap(AT_INPUT_NG_CNT))


def test_doc_examples():
    env = {
        'ansible_telemetry': ansible.AnsibleTelemetry(context_wrap(AT_INPUT)),
    }
    failed, total = doctest.testmod(ansible, globs=env)
    assert failed == 0



# -*- coding: UTF-8 -*-

from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from insights.client.support import InsightsSupport, registration_check
from insights.client.constants import InsightsConstants as constants
from mock.mock import Mock, patch


TEMP_TEST_REG_DIR = "/tmp/insights-client-registration"
TEMP_TEST_REG_DIR2 = "/tmp/redhat-access-insights-registration"


@patch("insights.client.support.subprocess")
@patch("insights.client.support.InsightsSupport._support_diag_dump")
def test_collect_support_info_support_diag_dump(support_diag_dump, subprocess):
    """
    collect_suppport_info_method callse _support_diag_dump method.
    """
    config = Mock()
    support = InsightsSupport(config)
    support.collect_support_info()

    support_diag_dump.assert_called_once_with()


@patch("insights.client.support.os.statvfs", **{"return_value.f_bavail": 1, "return_value.f_frsize": 1})
@patch("insights.client.support.Popen", **{"return_value.communicate.return_value": ("", "")})
@patch("insights.client.support.os.path.isfile", return_value=False)
@patch("insights.client.support.registration_check", return_value={})
@patch("insights.client.support.InsightsConnection")
def test_support_diag_dump_insights_connection(insights_connection, registration_check, isfile, popen, statvfs):
    """
    _support_diag_dump method instantiates InsightsConnection with InsightsConfig.
    """
    config = Mock(**{"return_value.auto_config": "", "proxy": "", "offline": False})
    support = InsightsSupport(config)
    support._support_diag_dump()

    insights_connection.assert_called_once_with(config)


@patch('insights.client.support.InsightsConnection')
@patch('insights.client.support.registration_check')
def test_registration_check_called_for_support(registration_check, InsightsConnection):
    '''
        Check registration when doing --support
    '''
    support = InsightsSupport(InsightsConfig(offline=False))
    support.collect_support_info()
    registration_check.assert_called_once()


@patch('insights.client.support.InsightsConnection')
@patch('insights.client.support.registration_check')
@patch('insights.client.support.InsightsConnection.test_connection')
def test_support_diag_dump_offline(test_connection, registration_check, InsightsConnection):
    '''
        Check registration when doing --support
    '''
    support = InsightsSupport(InsightsConfig(offline=True))
    support.collect_support_info()
    InsightsConnection.assert_not_called()
    registration_check.assert_not_called()
    test_connection.assert_not_called()


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_unregistered(write_to_disk, write_unregistered_file, write_registered_file, _, __):
    '''
    Ensure that connection function is called and files are written.
    '''
    config = Mock(base_url=None, legacy_upload=False)
    conn = InsightsConnection(config)
    conn.api_registration_check = Mock(return_value=False)
    assert registration_check(conn) is False
    conn.api_registration_check.assert_called_once()
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_called_once()


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
def test_registration_check_registered(write_unregistered_file, write_registered_file, _, __):
    '''
    Ensure that connection function is called and files are written.
    '''
    config = Mock(base_url=None, legacy_upload=False)
    conn = InsightsConnection(config)
    conn.api_registration_check = Mock(return_value=True)
    assert registration_check(conn) is True
    conn.api_registration_check.assert_called_once()
    write_registered_file.assert_called_once()
    write_unregistered_file.assert_not_called()


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_registration_check_registered_unreach(_, __):
    '''
    Ensure that connection function is called and files are written.
    '''
    config = Mock(base_url=None, legacy_upload=False)
    conn = InsightsConnection(config)
    conn.api_registration_check = Mock(return_value=None)
    assert registration_check(conn) is None
    conn.api_registration_check.assert_called_once()


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.api_registration_check", return_value=False)
@patch("insights.client.support.machine_id_exists", return_value=True)
@patch("os.path.exists", side_effect=lambda path: path == constants.registered_files[0])
def test_registration_check_registered_no_inventory(mock_os_path_exists, machine_id_exists, api_registration_check, get_proxies, _init_session):
    '''
    Test the registration check function when the system does not exist in Inventory,
    but the machine-id and .registered files are present locally on the system.
    '''
    config = Mock(base_url=None, legacy_upload=False)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert check is True


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=404, content='{"detail": "System with insights_id ID not found"}'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_legacy_unregistered_good_json(write_to_disk, write_unregistered_file, write_registered_file, _geturl, _proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    In this test insights-client has a machine-id file but is not registered in server,
    it checks if insights-client write the unregistered files when it notices is not registered.
    Ensure that connection function is called and data processed.
    When the system is not registered the server sends a 404 error with a json content.
    Check that when the client get this response the registered file and the machine-id files
    are removed and the unregistered file is created
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is False
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_called_once()
    write_to_disk.assert_called_once()


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=404, content='{Page Not Found}'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_legacy_unregistered_bad_json(write_to_disk, write_unregistered_file, write_registered_file, _geturl, _proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    Ensure that connection function is called and data processed.
    Check a 404 from a forward that doesn't contain a json a content.
    Check that when the client get this response nothing happens
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is False
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_not_called()
    write_to_disk.assert_not_called()


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=200, content='{"unregistered_at": "2019-04-10"}'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_legacy_registered_then_unregistered(write_to_disk, write_unregistered_file, write_registered_file, _geturl, _proxies, _init_session, _machine_id_exists, _generate_machine_id):
    '''
    This test checks if insights-client removes the registration file when it notices it is not
    registered through the API but has a machine-id file.
    Ensure that connection function is called and data processed.
    Legacy version responded with the unregistered_at as a json parameter.
    Check that when the client get this response the registered file and the machine-id files
    are removed and the unregistered file is created
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is False
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_called_once()


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=200, content='{"unregistered_at": null}'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
def test_registration_check_legacy_registered(write_unregistered_file, write_registered_file, _geturl, _proxy, _session, _machine_id_exists, _generate_machine_id):
    '''
    Ensure that connection function is called and data processed.
    When the systems is registered it get a 200 message with a json with the unregistered_at=null
    Check that when the client get this response the unregistered file is removed
    and the registered file and machine-id exists
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is True
    write_registered_file.assert_called_once()
    write_unregistered_file.assert_not_called()


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=502, content='zSDFasfghsRGH'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_legacy_bad_json(write_to_disk, write_unregistered_file, write_registered_file, _geturl, _proxy, _session, _machine_id_exists, _generate_machine_id):
    '''
    Ensure the function does not remove any file if a network error is encountered
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is False
    assert check['unreachable'] is True
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_not_called()
    write_to_disk.assert_not_called()


@patch("insights.client.connection.generate_machine_id", return_value='xxxxxx')
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.InsightsConnection.get", return_value=Mock(status_code=502, content='{"details": "Bad gateway"}'))
@patch('insights.client.support.write_registered_file')
@patch('insights.client.support.write_unregistered_file')
@patch('insights.client.support.write_to_disk')
def test_registration_check_legacy_bad_connection(write_to_disk, write_unregistered_file, write_registered_file, _geturl, _proxy, _session, _machine_id_exists, _generate_machine_id):
    '''
    Ensure the function does not remove any file if a network error is encountered
    '''
    config = Mock(base_url=None, legacy_upload=True)
    conn = InsightsConnection(config)
    check = registration_check(conn)
    assert isinstance(check, dict)
    assert check['status'] is False
    assert check['unreachable'] is True
    write_registered_file.assert_not_called()
    write_unregistered_file.assert_not_called()
    write_to_disk.assert_not_called()



"""
Limits configuration
====================

The limits files are normally available to rules as a list of LimitsConf
objects.  This combiner turns those into one set of data, and provides a
``find_all()`` method to search the rules from all the files.
"""

from insights.core.plugins import combiner
from insights.parsers.limits_conf import LimitsConf


@combiner(LimitsConf)
class AllLimitsConf(object):
    """
    Combiner for accessing all the limits configuration files.

    Attributes:
        domains(set): the set of domains found in all data files.
        limits(list): a list of the original LimitsConf parser results.
        rules(list): the entire list of rules.
    """
    def __init__(self, limits):
        rules = []
        domains = set([])
        self.limits = limits
        # Store the tuple of domain, type and item against the row of that
        # limit in our full list of rules.
        rule_tuples_found = {}
        rule_number = 0
        for limits in sorted(limits, key=lambda f: f.file_path):
            # The rule dictionary contains the file_path as a key for later.
            # Make sure that we keep only the last rule for this combination
            # of domain, type and item, according to https://access.redhat.com/solutions/199993
            for rule in limits.rules:
                rule_tuple = (rule['domain'], rule['type'], rule['item'])
                if rule_tuple in rule_tuples_found:
                    row_number = rule_tuples_found[rule_tuple]
                    rules[row_number] = rule
                else:
                    rules.append(rule)
                    rule_tuples_found[rule_tuple] = rule_number
                    rule_number += 1
            # The domains update is much easier :-)
            domains.update(limits.domains)

        self.rules = rules
        self.domains = domains
        super(AllLimitsConf, self).__init__()

    def find_all(self, **kwargs):
        """
        Find all the rules that match the given parameters.  We cheat a bit
        here and combine the results from the find_all() method from the
        original parsers.  Otherwise we'd have to reimplement the _matches
        method from the LimitsConf class.

        Examples:
            >>> data = limits
            >>> results = data.find_all(domain='nproc')
            >>> len(results)
            1
            >>> results[0]['domain']
            'nproc'

        Parameters:
            **kwargs(dict): key-value pairs for the search data.

        Returns:
            (list): a list of the rules matching the given keywords, as
                determined by the ``_matches()`` method in the ``LimitsConf``
                class.
        """
        return [match for data in self.limits for match in data.find_all(**kwargs)]



import time
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from mock.mock import patch, Mock, call
from unittest import TestCase
from pytest import raises
from insights.client.constants import InsightsConstants as constants

test_timestamp = '000000'
test_hostname = 'testhostname'
test_uuidhex = Mock(hex='000000')
test_archive_name = 'insights-testhostname-000000'
test_archive_dir = '/var/tmp/insights-client-000000/insights-testhostname-000000'
test_obfuscated_archive_dir = '/var/tmp/insights-client-000000/insights-000000-000000'
test_tmp_dir_path = '/var/tmp/insights-client-000000'
test_tmp_dir = 'insights-client-000000'


@patch('insights.client.archive.time.strftime', Mock(return_value=test_timestamp))
@patch('insights.client.archive.determine_hostname', Mock(return_value=test_hostname))
@patch('insights.client.archive.tempfile.mkdtemp')
@patch('insights.client.archive.atexit.register')
class TestInsightsArchive(TestCase):
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive')
    def test_init_archive(self, cleanup, register, mkdtemp):
        '''
        Verify archive is created with default parameters
        '''
        config = Mock()
        config.obfuscation_list = []
        archive = InsightsArchive(config)

        assert archive.config == config
        assert archive.tmp_dir
        assert archive.archive_dir is None
        assert archive.compressor == config.compressor
        assert archive.archive_name == test_archive_name

        cleanup.assert_called_once()
        mkdtemp.assert_has_calls(
            [call(dir=constants.insights_tmp_path, prefix=constants.insights_tmp_prefix + '-')]
        )
        register.assert_called_once()

    @patch('insights.client.archive.os.path.exists', return_value=False)
    @patch('insights.client.archive.os.makedirs')
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive')
    def test_create_archive_dir_default(self, _cleanup, mkdir, _exists, _, __):
        '''
        Verify archive_dir is created when it does not already exist
        '''
        config = Mock()
        config.obfuscation_list = None
        archive = InsightsArchive(config)
        archive.tmp_dir = test_tmp_dir_path
        result = archive.create_archive_dir()

        mkdir.assert_called_once_with(test_archive_dir, 0o700)
        # ensure the archive_dir is returned from the function
        assert result == test_archive_dir
        # ensure the class attr is set
        assert archive.archive_dir == test_archive_dir
        # ensure the retval and attr are the same
        assert result == archive.archive_dir

    @patch('insights.client.archive.os.listdir', return_value=[])
    @patch('insights.client.archive.shutil.rmtree')
    def test_tmp_dir_no_cleanup(self, rmtree, listdir, _, __):
        '''
        Verify cleanup doesn't act with no previous directories
        '''
        InsightsArchive(InsightsConfig())
        listdir.assert_called_once_with(constants.insights_tmp_path)
        rmtree.assert_not_called()

    @patch('insights.client.archive.os.listdir', return_value=[test_tmp_dir])
    @patch('insights.client.archive.shutil.rmtree')
    @patch('insights.client.archive.os.path.isdir', return_value=[True])
    @patch('insights.client.archive.time.time')
    def test_tmp_dir_no_cleanup_time(self, _timestamp, _isdir, rmtree, listdir, _, __):
        '''
        Verify cleanup doesn't act with previous directories that are not
        older that 1 day
        '''
        _timestamp.return_value = time.time()
        InsightsArchive(InsightsConfig())
        listdir.assert_called_once_with(constants.insights_tmp_path)
        rmtree.assert_not_called()

    @patch('insights.client.archive.os.listdir', return_value=[test_tmp_dir])
    @patch('insights.client.archive.shutil.rmtree')
    @patch('insights.client.archive.os.path.isdir', return_value=[True])
    @patch('insights.client.archive.os.path.getmtime')
    def test_tmp_dir_cleanup(self, _timestamp, _isdir, rmtree, listdir, _, __):
        # set the timestamp of the directory as it was create 2 days ago
        _timestamp.return_value = time.time() - 2 * 24 * 60 * 60
        InsightsArchive(InsightsConfig())
        listdir.assert_called_once_with(constants.insights_tmp_path)
        rmtree.assert_called_with(test_tmp_dir_path, True)

    @patch('insights.client.archive.uuid.uuid4', return_value=test_uuidhex)
    @patch('insights.client.archive.os.makedirs')
    def test_create_archive_dir_obfuscated(self, makedirs, uuid4, _, __):
        '''
        Verify archive_dir is created when it does not already exist
        '''
        config = Mock()
        config.obfuscation_list = ['hostname']
        with patch('insights.client.archive.os.path.exists', return_value=True):
            archive = InsightsArchive(config)
        # give this a discrete value so we can check the results
        archive.tmp_dir = test_tmp_dir_path

        with patch('insights.client.archive.os.path.exists', return_value=False):
            result = archive.create_archive_dir()
        makedirs.assert_called_once_with(test_obfuscated_archive_dir, 0o700)
        # ensure the archive_dir is returned from the function
        assert result == test_obfuscated_archive_dir
        # ensure the class attr is set
        assert archive.archive_dir == test_obfuscated_archive_dir
        # ensure the retval and attr are the same
        assert result == archive.archive_dir

    @patch('insights.client.archive.os.makedirs')
    @patch('insights.client.archive.os.path.exists', side_effect=[False, False])
    def test_create_archive_dir_defined_path_DNE(self, exists, makedirs, _, __):
        '''
        Verify archive_dir is created when the attr is defined but
        the path does not exist
        '''
        config = Mock()
        config.obfuscation_list = None
        archive = InsightsArchive(config)
        # give this a discrete value so we can check the results
        archive.tmp_dir = test_tmp_dir_path
        archive.archive_dir = test_archive_dir
        result = archive.create_archive_dir()
        exists.assert_has_calls([call(archive.archive_dir), call(test_archive_dir)])
        makedirs.assert_called_once_with(test_archive_dir, 0o700)
        # ensure the archive_dir is returned from the function
        assert result == test_archive_dir
        # ensure the class attr is set
        assert archive.archive_dir == test_archive_dir
        # ensure the retval and attr are the same
        assert result == archive.archive_dir

    @patch('insights.client.archive.os.makedirs')
    def test_create_archive_dir_undef_path_exists(self, makedirs, _, __):
        '''
        Verify archive_dir is not re-created when the attr is undefined but
        the path exists
        '''
        config = Mock()
        config.obfuscation_list = []
        with patch('insights.client.archive.os.path.exists', return_value=True):
            archive = InsightsArchive(config)
        # give this a discrete value so we can check the results
        archive.tmp_dir = test_tmp_dir_path
        with patch('insights.client.archive.os.path.exists', return_value=True) as exists:
            result = archive.create_archive_dir()
        makedirs.assert_not_called()
        exists.assert_called_once_with(test_archive_dir)
        # ensure the archive_dir is returned from the function
        assert result == test_archive_dir
        # ensure the class attr is set
        assert archive.archive_dir == test_archive_dir
        # ensure the retval and attr are the same
        assert result == archive.archive_dir

    @patch('insights.client.archive.os.makedirs')
    def test_create_archive_dir_defined_path_exists(self, makedirs, _, __):
        '''
        When archive_dir is defined and exists, simply return the
        class attr and do not attempt to create it
        '''
        with patch('insights.client.archive.os.path.exists', return_value=True):
            archive = InsightsArchive(InsightsConfig())
        # give this a discrete value so we can check the results
        archive.tmp_dir = '/var/tmp/test'
        archive.archive_dir = test_archive_dir
        with patch('insights.client.archive.os.path.exists', return_value=True) as exists:
            result = archive.create_archive_dir()
        makedirs.assert_not_called()
        exists.assert_called_once_with(archive.archive_dir)
        # ensure the archive_dir is returned from the function
        assert result == test_archive_dir
        # ensure the class attr is set
        assert archive.archive_dir == test_archive_dir
        # ensure the retval and attr are the same
        assert result == archive.archive_dir

    @patch(
        'insights.client.archive.InsightsArchive.create_archive_dir', return_value=test_archive_dir
    )
    @patch('insights.client.archive.os.path.join', Mock())
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive')
    def test_get_full_archive_path(self, create_archive_dir, cleanup, _, __):
        '''
        Verify create_archive_dir is called when calling get_full_archive_path
        '''
        archive = InsightsArchive(InsightsConfig())
        archive.get_full_archive_path('test')
        create_archive_dir.assert_called_once()

    @patch('insights.client.archive.os.path.isdir', return_value=False)
    @patch('insights.client.archive.shutil.copytree', return_value=None)
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive', return_value=None)
    @patch(
        'insights.client.archive.InsightsArchive.create_archive_dir', return_value=test_archive_dir
    )
    def test_copy_dir(self, create_archive_dir, _1, _2, _3, _4, _5):
        '''
        Verify create_archive_dir is called when calling copy_dir
        '''
        archive = InsightsArchive(InsightsConfig())
        archive.copy_dir('test')
        create_archive_dir.assert_called_once()

    @patch('insights.client.archive.shutil.copyfile')
    @patch('insights.client.archive.os.path.isdir', Mock())
    @patch('insights.client.archive.os.path.exists', return_value=True)
    def test_keep_archive(self, path_exists, copyfile, _, __):
        archive = InsightsArchive(InsightsConfig())
        archive.tar_file = '/var/tmp/insights-archive-test.tar.gz'
        archive.keep_archive_dir = '/var/tmp/test-archive'
        archive.storing_archive()
        copyfile.assert_called_once_with(
            archive.tar_file, '/var/tmp/test-archive/insights-archive-test.tar.gz'
        )

    @patch('insights.client.archive.shutil.copyfile', side_effect=OSError)
    @patch('insights.client.archive.os.path.join', Mock())
    @patch('insights.client.archive.os.path.isdir', Mock())
    @patch('insights.client.archive.os.path.basename', Mock())
    @patch('insights.client.archive.logger')
    @patch('insights.client.archive.os.path.exists', return_value=True)
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive', Mock())
    def test_keep_archive_err_during_copy(self, path_exists, logger, copyfile, _, __):
        archive = InsightsArchive(InsightsConfig())
        archive.archive_stored = '/var/tmp/test-archive/test-store-archive'
        archive.keep_archive_dir = '/var/tmp/test-archive'
        with raises(Exception):
            archive.storing_archive()
        logger.error.assert_called_once_with(
            'ERROR: Could not stored archive to %s', archive.archive_stored
        )

    @patch('insights.client.archive.os.makedirs', side_effect=OSError)
    @patch('insights.client.archive.os.path.exists', side_effect=[False])
    @patch('insights.client.archive.os.path.join', Mock())
    @patch('insights.client.archive.os.path.isdir', Mock())
    @patch('insights.client.archive.os.path.basename', Mock())
    @patch('insights.client.archive.logger')
    @patch('insights.client.archive.InsightsArchive.cleanup_previous_archive', Mock())
    def test_keep_arhive_err_creating_directory(self, logger, path_exists, mkdir, _, __):
        archive = InsightsArchive(InsightsConfig())
        archive.keep_archive_dir = '/var/tmp/test-archive'
        with raises(Exception):
            archive.storing_archive()
        logger.error.assert_called_with('ERROR: Could not create %s', archive.keep_archive_dir)



import tempfile

from mock.mock import call, patch
from pytest import mark

import insights.client.schedule as sched
from insights.client.config import InsightsConfig


def test_get_schedule_cron():
    target = tempfile.mktemp()
    config = InsightsConfig()
    with tempfile.NamedTemporaryFile() as source:
        schedule = sched.get_scheduler(config, source.name, target)
        assert isinstance(schedule, sched.InsightsSchedulerCron)


def test_schedule_cron():
    target = tempfile.mktemp()
    config = InsightsConfig()
    with tempfile.NamedTemporaryFile() as source:
        schedule = sched.InsightsSchedulerCron(config, source.name, target)
        assert not schedule.active
        assert schedule.schedule()
        assert schedule.active
        schedule.remove_scheduling()
        assert not schedule.active


def test_failed_removal():
    """
    Just verifying that trying to remove scheduling does not raise an exception
    """
    target = tempfile.mktemp()
    config = InsightsConfig()
    with tempfile.NamedTemporaryFile() as source:
        schedule = sched.get_scheduler(config, source.name, target)
        schedule.remove_scheduling()


def test_get_scheduler_systemd():
    config = InsightsConfig()
    schedule = sched.get_scheduler(config, "no cron")
    assert isinstance(schedule, sched.InsightsSchedulerSystemd)


@patch("insights.client.schedule.run_command_get_output", return_value={"status": 0, "output": "LoadState=loaded\n"})
def test_init_systemd_calls(run_command_get_output):
    sched.InsightsSchedulerSystemd()
    calls = (
        call("systemctl show --property LoadState insights-client.timer"),
        call("systemctl show --property LoadState insights-client-checkin.timer"),
    )
    run_command_get_output.assert_has_calls(calls)


@mark.parametrize(("load_states", "loaded_timers"), (
    (("loaded", "loaded"), ("insights-client", "insights-client-checkin")),
    (("loaded", "not-found"), ("insights-client",)),
    (("not-found", "not-found"), ()),
))
@patch("insights.client.schedule.run_command_get_output")
def test_init_systemd_loaded_timers(run_command_get_output, load_states, loaded_timers):
    run_command_get_output.side_effect = (
        {"status": 0, "output": "LoadState=%s\n" % load_state}
        for load_state in load_states
    )
    scheduler = sched.InsightsSchedulerSystemd()
    assert scheduler.loaded_timers == loaded_timers


@mark.parametrize(("side_effect"), (({"status": 0, "output": "LoadState=loaded\n"}, OSError), (OSError,)))
@patch("insights.client.schedule.run_command_get_output")
def test_init_systemd_error(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect
    scheduler = sched.InsightsSchedulerSystemd()
    assert scheduler.loaded_timers is None


@patch("insights.client.schedule.run_command_get_output", return_value={"status": 0, "output": "LoadState=loaded\n"})
def test_init_calls(run_command_get_output):
    sched.InsightsSchedulerSystemd()
    calls = (
        call("systemctl show --property LoadState insights-client.timer"),
        call("systemctl show --property LoadState insights-client-checkin.timer"),
    )
    run_command_get_output.assert_has_calls(calls)


@mark.parametrize(("outputs", "calls"), (
    (
        ("LoadState=loaded\n", "LoadState=loaded\n", "", "", "", ""),
        (
            call("systemctl enable --now insights-client.timer"),
            call("systemctl enable --now insights-client-checkin.timer"),
            call("systemctl is-enabled insights-client.timer"),
            call("systemctl is-enabled insights-client-checkin.timer"),
        ),
    ),
    (
        ("LoadState=loaded\n", "LoadState=not-found\n", "", ""),
        (
            call("systemctl enable --now insights-client.timer"),
            call("systemctl is-enabled insights-client.timer"),
        ),
    ),
    (
        ("LoadState=not-found\n", "LoadState=not-found\n", "", ""),
        (),
    ),
))
@patch("insights.client.schedule.run_command_get_output")
def test_schedule_systemd_calls(run_command_get_output, outputs, calls):
    run_command_get_output.side_effect = ({"status": 0, "output": output} for output in outputs)

    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    schedule.schedule()
    run_command_get_output.assert_has_calls(calls)


@mark.parametrize(("outputs",), (
    (("LoadState=loaded\n", "LoadState=loaded\n", "", "", "", ""),),
    (("LoadState=loaded\n", "LoadState=not-found\n", "", ""),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_schedule_systemd_success(run_command_get_output, outputs):
    run_command_get_output.side_effect = ({"status": 0, "output": output} for output in outputs)
    schedule = sched.InsightsSchedulerSystemd()
    scheduled = schedule.schedule()
    assert scheduled is True


@mark.parametrize(("side_effect",), (
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
        {"status": 0, "output": ""},
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
        {"status": 1, "output": ""},
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=not-found\n"},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_schedule_systemd_inactive(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect

    schedule = sched.InsightsSchedulerSystemd()
    scheduled = schedule.schedule()
    assert scheduled is False


@mark.parametrize(("side_effect",), (
    ((
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        OSError
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        OSError
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        OSError
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_schedule_systemd_call_error(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect

    schedule = sched.InsightsSchedulerSystemd()
    scheduled = schedule.schedule()
    assert scheduled is None


@patch(
    "insights.client.schedule.run_command_get_output",
    return_value={"status": 0, "output": "LoadState=not-found\n"},
)
def test_schedule_systemd_not_found(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    scheduled = schedule.schedule()
    assert scheduled == {}
    run_command_get_output.assert_not_called()


@patch("insights.client.schedule.run_command_get_output", side_effect=OSError)
def test_schedule_systemd_init_error(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    scheduled = schedule.schedule()
    assert scheduled is None
    run_command_get_output.assert_not_called()


@mark.parametrize(("outputs", "calls"), (
    (
        ("LoadState=loaded\n", "LoadState=loaded\n", "", "", "", ""),
        (
            call("systemctl disable --now insights-client.timer"),
            call("systemctl disable --now insights-client-checkin.timer"),
            call("systemctl is-enabled insights-client.timer"),
            call("systemctl is-enabled insights-client-checkin.timer"),
        ),
    ),
    (
        ("LoadState=loaded\n", "LoadState=not-found\n", "", ""),
        (
            call("systemctl disable --now insights-client.timer"),
            call("systemctl is-enabled insights-client.timer"),
        ),
    ),
    (
        ("LoadState=not-found\n", "LoadState=not-found\n", "", ""),
        (),
    ),
))
@patch("insights.client.schedule.run_command_get_output")
def test_remove_scheduling_systemd_calls(run_command_get_output, outputs, calls):
    run_command_get_output.side_effect = ({"status": 0, "output": output} for output in outputs)

    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    schedule.remove_scheduling()
    run_command_get_output.assert_has_calls(calls)


@mark.parametrize(("side_effect",), (
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
        {"status": 1, "output": ""},
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=not-found\n"},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""},
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_remove_scheduling_systemd_success(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect
    schedule = sched.InsightsSchedulerSystemd()
    unscheduled = schedule.remove_scheduling()
    assert unscheduled is True


@mark.parametrize(("side_effect",), (
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=not-found\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_remove_scheduling_systemd_active(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect
    schedule = sched.InsightsSchedulerSystemd()
    unscheduled = schedule.remove_scheduling()
    assert unscheduled is False


@mark.parametrize(("side_effect",), (
    ((
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        OSError
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        OSError
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        {"status": 0, "output": ""},
        OSError
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_remove_scheduling_systemd_error(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect

    schedule = sched.InsightsSchedulerSystemd()
    unscheduled = schedule.remove_scheduling()
    assert unscheduled is None


@patch(
    "insights.client.schedule.run_command_get_output",
    return_value={"status": 0, "output": "LoadState=not-found\n"},
)
def test_remove_scheduling_systemd_not_found(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    unscheduled = schedule.remove_scheduling()
    assert unscheduled == {}
    run_command_get_output.assert_not_called()


@patch("insights.client.schedule.run_command_get_output", side_effect=OSError)
def test_remove_scheduling_systemd_init_error(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    unscheduled = schedule.remove_scheduling()
    assert unscheduled is None
    run_command_get_output.assert_not_called()


@mark.parametrize(("outputs", "calls"), (
    (
        ("LoadState=loaded\n", "LoadState=loaded\n", "", "", "", ""),
        (
            call("systemctl is-enabled insights-client.timer"),
            call("systemctl is-enabled insights-client-checkin.timer"),
        ),
    ),
    (
        ("LoadState=loaded\n", "LoadState=not-found\n", "", ""),
        (
            call("systemctl is-enabled insights-client.timer"),
        ),
    ),
    (
        ("LoadState=not-found\n", "LoadState=not-found\n", "", ""),
        (),
    ),
))
@patch("insights.client.schedule.run_command_get_output")
def test_active_systemd_calls(run_command_get_output, outputs, calls):
    run_command_get_output.side_effect = ({"status": 0, "output": output} for output in outputs)

    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()
    schedule.active

    run_command_get_output.assert_has_calls(calls)


@mark.parametrize(("outputs",), (
    (("LoadState=loaded\n", "LoadState=loaded\n", "", ""),),
    (("LoadState=loaded\n", "LoadState=not-found\n", ""),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_active_systemd_active(run_command_get_output, outputs):
    run_command_get_output.side_effect = ({"status": 0, "output": output} for output in outputs)
    schedule = sched.InsightsSchedulerSystemd()
    assert schedule.active is True


@mark.parametrize(("side_effect",), (
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        {"status": 1, "output": ""}
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 1, "output": ""},
        {"status": 0, "output": ""}
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 1, "output": ""},
        {"status": 1, "output": ""}
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=not-found\n"},
        {"status": 1, "output": ""},
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_active_systemd_inactive(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect
    schedule = sched.InsightsSchedulerSystemd()
    assert schedule.active is False


@mark.parametrize(("side_effect",), (
    ((
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        OSError,
    ),),
    ((
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": "LoadState=loaded\n"},
        {"status": 0, "output": ""},
        OSError
    ),),
))
@patch("insights.client.schedule.run_command_get_output")
def test_active_systemd_error(run_command_get_output, side_effect):
    run_command_get_output.side_effect = side_effect
    schedule = sched.InsightsSchedulerSystemd()
    assert schedule.active is None


@patch(
    "insights.client.schedule.run_command_get_output",
    side_effect=(
        {"status": 0, "output": "LoadState=not-found\n"},
        {"status": 0, "output": "LoadState=not-found\n"},
    ),
)
def test_active_systemd_not_found(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    assert schedule.active == {}
    run_command_get_output.assert_not_called()


@patch("insights.client.schedule.run_command_get_output", side_effect=OSError)
def test_active_systemd_init_error(run_command_get_output):
    schedule = sched.InsightsSchedulerSystemd()
    run_command_get_output.reset_mock()

    assert schedule.active is None
    run_command_get_output.assert_not_called()



from insights.parsers.sysconfig import LibvirtGuestsSysconfig
from insights.tests import context_wrap

SYSCONFIG_LIBVIRT_GUESTS = """
# URIs to check for running guests
# example: URIS='default xen:/// vbox+tcp://host/system lxc:///'
#URIS=default

# action taken on host boot
# - start   all guests which were running on shutdown are started on boot
#           regardless on their autostart settings
# - ignore  libvirt-guests init script won't start any guest on boot, however,
#           guests marked as autostart will still be automatically started by
#           libvirtd
ON_BOOT=ignore

# Number of seconds to wait between each guest start. Set to 0 to allow
# parallel startup.
#START_DELAY=0

# action taken on host shutdown
# - suspend   all running guests are suspended using virsh managedsave
# - shutdown  all running guests are asked to shutdown. Please be careful with
#             this settings since there is no way to distinguish between a
#             guest which is stuck or ignores shutdown requests and a guest
#             which just needs a long time to shutdown. When setting
#             ON_SHUTDOWN=shutdown, you must also set SHUTDOWN_TIMEOUT to a
#             value suitable for your guests.
#ON_SHUTDOWN=suspend

# If set to non-zero, shutdown will suspend guests concurrently. Number of
# guests on shutdown at any time will not exceed number set in this variable.
#PARALLEL_SHUTDOWN=0

# Number of seconds we're willing to wait for a guest to shut down. If parallel
# shutdown is enabled, this timeout applies as a timeout for shutting down all
# guests on a single URI defined in the variable URIS. If this is 0, then there
# is no time out (use with caution, as guests might not respond to a shutdown
# request). The default value is 300 seconds (5 minutes).
#SHUTDOWN_TIMEOUT=300

# If non-zero, try to bypass the file system cache when saving and
# restoring guests, even though this may give slower operation for
# some file systems.
#BYPASS_CACHE=0

# If non-zero, try to sync guest time on domain resume. Be aware, that
# this requires guest agent with support for time synchronization
# running in the guest. For instance, qemu-ga doesn't support guest time
# synchronization on Windows guests, but Linux ones. By default, this
# functionality is turned off.
#SYNC_TIME=1
""".strip()


def test_sysconfig_libvirt_guests():
    libvirt_guests_sysconfig = LibvirtGuestsSysconfig(context_wrap(SYSCONFIG_LIBVIRT_GUESTS))
    assert 'ON_BOOT' in libvirt_guests_sysconfig
    assert libvirt_guests_sysconfig.get('ON_BOOT') == 'ignore'
    assert libvirt_guests_sysconfig.get('SYNC_TIME') is None
